; MIT License

; Copyright (C) 2022 Z80Enjoyers / Luis Jesús Marhuenda Tendero (@ljmt1) / Alberto Vegara Belmonte (@avb52-ua) / Samuel Benjamín Oliva Bulpitt (@samuel0122)

; Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal 
; in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
; copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

; The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS 
; FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
; WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

;;======================================================================================================
.module Animations_Manager
;;======================================================================================================
.area _CODE
;;======================================================================================================
.include "cpctelera.h.s"
.include "man/animations.h.s"
.include "assets/sprites.inc"
.include "animations.h.s"

;;======================================================================================================

;;======================================================================================================
;;  Animations Array
;;
;;  Preconditions:
;;      0000 Right Slow
;;      0001 Left
;;      0010 Right Fast
;;      0011 --- nullptr
;;      0100 Down
;;      0101 Down-Left
;;      0110 Down-Right
;;      0111 --- nullptr
;;      1000 Up
;;      1001 Up-Left
;;      1010 Up-Right
;;      1011 --- nullptr
;;
_animations_array:
    .dw     _player_walking_right_slow
    .dw     _player_walking_left
    .dw     _player_walking_right_fast
    .dw     nullptr
    .dw     _player_flying_down
    .dw     _player_flying_down_left
    .dw     _player_flying_down_right
    .dw     nullptr
    .dw     _player_flying_up
    .dw     _player_flying_up_left
    .dw     _player_flying_up_right

;;======================================================================================================
;;  Manager Animations Get Animation Pointer in HL
;;  
;;  Input:
;;      - B = Animation Status
;;  Output:
;;      - HL = Pointer to animation
;;
man_animations_get_animPtr_HL::
    ld hl, #_animations_array
    ld a, b
    add a
    add_hl_a

    ld a, (hl)
    inc hl
    ld h, (hl)
    ld l, a

    ret

;;======================================================================================================
;; Player Animations
;;
_player_walking_right_slow::
    .db     player_anim_counter
    .dw     _sprt_player_00
    .db     player_anim_counter
    .dw     _sprt_player_01
    .db     nullptr
    .dw     _player_walking_right_slow

_player_walking_right_fast::
    .db     player_anim_counter
    .dw     _sprt_player_02
    .db     player_anim_counter
    .dw     _sprt_player_03
    .db     nullptr
    .dw     _player_walking_right_fast

_player_walking_left::
    .db     player_anim_counter
    .dw     _sprt_player_04
    .db     player_anim_counter
    .dw     _sprt_player_05
    .db     nullptr
    .dw     _player_walking_left

_player_flying_down::
    .db     player_anim_counter
    .dw     _sprt_player_06
    .db     player_anim_counter
    .dw     _sprt_player_07
    .db     nullptr
    .dw     _player_flying_down

_player_flying_down_left::
    .db     player_anim_counter
    .dw     _sprt_player_08
    .db     player_anim_counter
    .dw     _sprt_player_09
    .db     nullptr
    .dw     _player_flying_down_left

_player_flying_down_right::
    .db     player_anim_counter
    .dw     _sprt_player_06
    .db     player_anim_counter
    .dw     _sprt_player_07
    .db     nullptr
    .dw     _player_flying_down_right

_player_flying_up::
    .db     player_anim_counter
    .dw     _sprt_player_10
    .db     player_anim_counter
    .dw     _sprt_player_11
    .db     nullptr
    .dw     _player_flying_up

_player_flying_up_left::
    .db     player_anim_counter
    .dw     _sprt_player_12
    .db     player_anim_counter
    .dw     _sprt_player_13
    .db     nullptr
    .dw     _player_flying_up_left

_player_flying_up_right::
    .db     player_anim_counter
    .dw     _sprt_player_10
    .db     player_anim_counter
    .dw     _sprt_player_11
    .db     nullptr
    .dw     _player_flying_up_right

;;======================================================================================================
;; Small Fireball Enemy Animations
;;
_small_fireball_enemy::
    .db     fireball_anim_counter
    .dw     _sprt_small_fireball_0
    .db     fireball_anim_counter
    .dw     _sprt_small_fireball_1
    .db     fireball_anim_counter
    .dw     _sprt_small_fireball_2
    .db     fireball_anim_counter
    .dw     _sprt_small_fireball_3
    .db     fireball_anim_counter
    .dw     _sprt_small_fireball_2
    .db     fireball_anim_counter
    .dw     _sprt_small_fireball_1
    .db     nullptr
    .dw     _small_fireball_enemy

;;======================================================================================================
;; Medium Fireball Enemy Animations
;;
_medium_fireball_enemy::
    .db     fireball_anim_counter
    .dw     _sprt_medium_fireball_0
    .db     fireball_anim_counter
    .dw     _sprt_medium_fireball_1
    .db     fireball_anim_counter
    .dw     _sprt_medium_fireball_2
    .db     fireball_anim_counter
    .dw     _sprt_medium_fireball_3
    .db     fireball_anim_counter
    .dw     _sprt_medium_fireball_2
    .db     fireball_anim_counter
    .dw     _sprt_medium_fireball_1
    .db     nullptr
    .dw     _medium_fireball_enemy

;;======================================================================================================
;; Big Fireball Enemy Animations
;;
_big_fireball_enemy::
    .db     fireball_anim_counter
    .dw     _sprt_big_fireball_0
    .db     fireball_anim_counter
    .dw     _sprt_big_fireball_1
    .db     fireball_anim_counter
    .dw     _sprt_big_fireball_2
    .db     fireball_anim_counter
    .dw     _sprt_big_fireball_3
    .db     fireball_anim_counter
    .dw     _sprt_big_fireball_2
    .db     fireball_anim_counter
    .dw     _sprt_big_fireball_1
    .db     nullptr
    .dw     _big_fireball_enemy

;;======================================================================================================
;; 2 Fireballs-H Enemy Animations
;;
_fireballs_h_enemy::
    .db     fireball_anim_counter
    .dw     _sprt_fireballs_h_0
    .db     fireball_anim_counter
    .dw     _sprt_fireballs_h_1
    .db     fireball_anim_counter
    .dw     _sprt_fireballs_h_2
    .db     fireball_anim_counter
    .dw     _sprt_fireballs_h_3
    .db     fireball_anim_counter
    .dw     _sprt_fireballs_h_2
    .db     fireball_anim_counter
    .dw     _sprt_fireballs_h_1
    .db     nullptr
    .dw     _fireballs_h_enemy

_fireballs_h_larger_enemy::
    .db     fireball_anim_counter
    .dw     _sprt_fireballs_h_larger_0
    .db     fireball_anim_counter
    .dw     _sprt_fireballs_h_larger_1
    .db     fireball_anim_counter
    .dw     _sprt_fireballs_h_larger_2
    .db     fireball_anim_counter
    .dw     _sprt_fireballs_h_larger_3
    .db     fireball_anim_counter
    .dw     _sprt_fireballs_h_larger_2
    .db     fireball_anim_counter
    .dw     _sprt_fireballs_h_larger_1
    .db     nullptr
    .dw     _fireballs_h_larger_enemy

;;======================================================================================================
;; 2 Fireballs-V Enemy Animations
;;
_fireballs_v_enemy::
    .db     fireball_anim_counter
    .dw     _sprt_fireballs_v_0
    .db     fireball_anim_counter
    .dw     _sprt_fireballs_v_1
    .db     fireball_anim_counter
    .dw     _sprt_fireballs_v_2
    .db     fireball_anim_counter
    .dw     _sprt_fireballs_v_3
    .db     fireball_anim_counter
    .dw     _sprt_fireballs_v_2
    .db     fireball_anim_counter
    .dw     _sprt_fireballs_v_1
    .db     nullptr
    .dw     _fireballs_v_enemy


;;======================================================================================================
;; Level Message Animations
;;
level_message_anim::
    .db     level_message_anim_counter
    .dw     _spr_level_message_0
    .db     level_message_anim_counter
    .dw     _spr_level_message_1
    .db     level_message_anim_counter
    .dw     _spr_level_message_0
    .db     level_message_anim_counter
    .dw     _spr_level_message_1
    .db     level_message_anim_counter
    .dw     _spr_level_message_2
    .db     level_message_anim_counter
    .dw     _spr_level_message_3
    .db     level_message_anim_counter
    .dw     _spr_level_message_4
    .db     level_message_anim_counter
    .dw     _spr_level_message_5
    .db     level_message_anim_counter
    .dw     _spr_level_message_6
    .db     level_message_anim_counter
    .dw     _spr_level_message_1
    .db     level_message_anim_counter
    .dw     _spr_level_message_0
    .db     level_message_anim_counter
    .dw     _spr_level_message_1
    .db     level_message_anim_counter
    .dw     _spr_level_message_0
    .db     nullptr
    .dw     level_message_anim


;;======================================================================================================
;; Numbers for Level Message Animations
;;

.macro defineNumberAnim Anim0, Anim1
    .db     level_message_anim_counter
    .dw     Anim0
    .db     level_message_anim_counter
    .dw     Anim1
    .db     level_message_anim_counter
    .dw     Anim0
    .db     level_message_anim_counter
    .dw     Anim1
    .db     level_message_anim_counter
    .dw     Anim1
    .db     level_message_anim_counter
    .dw     Anim1
    .db     level_message_anim_counter
    .dw     Anim1
    .db     level_message_anim_counter
    .dw     Anim1
    .db     level_message_anim_counter
    .dw     Anim1
    .db     level_message_anim_counter
    .dw     Anim0
    .db     level_message_anim_counter
    .dw     Anim0
    .db     level_message_anim_counter
    .dw     Anim1
    .db     level_message_anim_counter
    .dw     Anim0
.endm

;;
;; Level 1
;;
number1_message_anim::
    defineNumberAnim _spr_number1_message_0, _spr_number1_message_1
    .db     nullptr
    .dw     number1_message_anim


;;
;; Level 2
;;
number2_message_anim::
    defineNumberAnim _spr_number2_message_0, _spr_number2_message_1
    .db     nullptr
    .dw     number2_message_anim


;;
;; Level 3
;;
number3_message_anim::
    defineNumberAnim _spr_number3_message_0, _spr_number3_message_1
    .db     nullptr
    .dw     number3_message_anim


;;
;; Level 4
;;
number4_message_anim::
    defineNumberAnim _spr_number4_message_0, _spr_number4_message_1
    .db     nullptr
    .dw     number4_message_anim


;;
;; Level 5
;;
number5_message_anim::
    defineNumberAnim _spr_number5_message_0, _spr_number5_message_1
    .db     nullptr
    .dw     number5_message_anim



;;
;; You wont escape
;;
you_wont_escape_message_anim::
    .db     level_message_anim_counter
    .dw     _spr_you_wont_escape_message_0
    .db     level_message_anim_counter
    .dw     _spr_you_wont_escape_message_1
    .db     nullptr
    .dw     you_wont_escape_message_anim


;;
;; NOO!!!
;;
nooo_message_anim::
    .db     level_message_anim_counter
    .dw     _spr_nooo_message_0
    .db     level_message_anim_counter
    .dw     _spr_nooo_message_1
    .db     nullptr
    .dw     nooo_message_anim