; MIT License

; Copyright (C) 2022 Z80Enjoyers / Luis Jesús Marhuenda Tendero (@ljmt1) / Alberto Vegara Belmonte (@avb52-ua) / Samuel Benjamín Oliva Bulpitt (@samuel0122)

; Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal 
; in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
; copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

; The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS 
; FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
; WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Entity:
;;       0 - type (1)
;;       1 - cmps (1)
;;       2 - x (1)
;;       3 - y (1)
;;       4 - w (1)
;;       5 - h (1)
;;       6 - vx (1)
;;       7 - vy (1)
;;       8 - sprite (2)
;;      10 - collides_against [type] (1)
;;      11 - animations (2)
;;      13 - anim_count (1)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;======================================================================================================
.module Entity_Templates_Manager
;;======================================================================================================
.area _CODE
;;======================================================================================================
.include "man/entity.h.s"
.include "sys/ai.h.s"
.include "assets/sprites.inc"
.include "man/animations.h.s"
.include "man/macros.h.s"
;;======================================================================================================

;;------------------------------------------------------------------------------------------------------
;; Entity Template
;;
tmpl_player::
    .db #e_type_player                                                      ; type
    .db #(e_cmp_render | e_cmp_input | e_cmp_physics | e_cmp_collider | e_cmp_animated)      ; cmps
    .db #10, #150                                                           ; x, y
    .db #SPRT_PLAYER_W, #SPRT_PLAYER_H                                      ; w, h
    .db #0, #0                                                              ; vx, vy
    .dw #_sprt_player_00                                                    ; sprite of the entity
    .db #e_type_enemy                                                       ; collides_against
    .db #0                                                                  ; e_colliding_counter
    .db #2                                                                  ; Lifes of the entity
    .dw _player_walking_right_slow                                          ; Animations
    .db player_anim_counter                                                 ; Anim Counter
    .db #0                                                                  ; Anim Status
    .dw #0xAFBE                                                             ; ai function
    .db #0xF1                                                               ; ai counter
    .dw #0                                                              ; prev pointer
CheckSize tmpl_player, sizeof_e


;;------------------------------------------------------------------------------------------------------
;; Enemy Template
;;
tmpl_small_fireball::
    .db #e_type_enemy                                       ; type
    .db #(e_cmp_render | e_cmp_collider | e_cmp_physics | e_cmp_animated | e_cmp_ai)    ; cmps
    .db #0, #0                                              ; x, y
    .db #SPRT_SMALL_FIREBALL_W, #SPRT_SMALL_FIREBALL_H      ; w, h
    .db #-1, #0                                             ; vx, vy
    .dw #_sprt_small_fireball_0                             ; sprite of the entity
    .db #0                                                  ; collides_against
    .db #0                                                  ; e_colliding_counter
    .db #0                                                  ; Lifes of the entity
    .dw #_small_fireball_enemy                              ; Animations
    .db #fireball_anim_counter                              ; Anim Counter
    .db #0                                                  ; Anim Status
    .dw #0                                                  ; ai function
    .db #0                                                  ; ai counter
    .dw #0                                                              ; prev pointer
CheckSize tmpl_small_fireball, sizeof_e

tmpl_medium_fireball::
    .db #e_type_enemy                                       ; type
    .db #(e_cmp_render | e_cmp_collider | e_cmp_physics | e_cmp_animated | e_cmp_ai)    ; cmps
    .db #0, #0                                              ; x, y
    .db #SPRT_MEDIUM_FIREBALL_W, #SPRT_MEDIUM_FIREBALL_H    ; w, h
    .db #-1, #0                                             ; vx, vy
    .dw #_sprt_medium_fireball_0                            ; sprite of the entity
    .db #0                                                  ; collides_against
    .db #0                                                  ; e_colliding_counter
    .db #2                                                  ; Lifes of the entity
    .dw #_medium_fireball_enemy                             ; Animations
    .db #fireball_anim_counter                              ; Anim Counter
    .db #0                                                  ; Anim Status
    .dw #0                                                              ; ai function
    .db #0                                                              ; ai counter
    .dw #0                                                              ; prev pointer
CheckSize tmpl_medium_fireball, sizeof_e

tmpl_big_fireball::
    .db #e_type_enemy                                       ; type
    .db #(e_cmp_render | e_cmp_collider | e_cmp_physics | e_cmp_animated | e_cmp_ai)    ; cmps
    .db #0, #0                                              ; x, y
    .db #SPRT_BIG_FIREBALL_W, #SPRT_BIG_FIREBALL_H          ; w, h
    .db #-1, #0                                             ; vx, vy
    .dw #_sprt_big_fireball_0                                      ; sprite of the entity
    .db #0                                                  ; collides_against
    .db #0                                                  ; e_colliding_counter
    .db #2                                                  ; Lifes of the entity
    .dw #_big_fireball_enemy                                    ; Animations
    .db #fireball_anim_counter                          ; Anim Counter
    .db #0                                                  ; Anim Status
    .dw #0                                                              ; ai function
    .db #0                                                              ; ai counter
    .dw #0                                                              ; prev pointer
CheckSize tmpl_big_fireball, sizeof_e


tmpl_fireballs_h::
    .db #e_type_enemy                                       ; type
    .db #(e_cmp_render | e_cmp_collider | e_cmp_physics | e_cmp_animated | e_cmp_ai)    ; cmps
    .db #0, #0                                              ; x, y
    .db #SPRT_FIREBALLS_H_W, #SPRT_FIREBALLS_H_H            ; w, h
    .db #-1, #0                                             ; vx, vy
    .dw #_sprt_fireballs_h_0                                      ; sprite of the entity
    .db #0                                                  ; collides_against
    .db #0                                                  ; e_colliding_counter
    .db #2                                                  ; Lifes of the entity
    .dw #_fireballs_h_enemy                                    ; Animations
    .db #fireball_anim_counter                          ; Anim Counter
    .db #0                                                  ; Anim Status
    .dw #0                                                              ; ai function
    .db #0                                                              ; ai counter
    .dw #0                                                              ; prev pointer
CheckSize tmpl_fireballs_h, sizeof_e

tmpl_fireballs_h_larger::
    .db #e_type_enemy                                       ; type
    .db #(e_cmp_render | e_cmp_collider | e_cmp_physics | e_cmp_animated | e_cmp_ai)    ; cmps
    .db #0, #0                                              ; x, y
    .db #SPRT_FIREBALLS_H_LARGER_0_W, #SPRT_FIREBALLS_H_LARGER_0_H            ; w, h
    .db #-1, #0                                             ; vx, vy
    .dw #_sprt_fireballs_h_larger_0                                      ; sprite of the entity
    .db #0                                                  ; collides_against
    .db #0                                                  ; e_colliding_counter
    .db #2                                                  ; Lifes of the entity
    .dw #_fireballs_h_larger_enemy                                    ; Animations
    .db #fireball_anim_counter                          ; Anim Counter
    .db #0                                                  ; Anim Status
    .dw #0                                                              ; ai function
    .db #0                                                              ; ai counter
    .dw #0                                                              ; prev pointer
CheckSize tmpl_fireballs_h_larger, sizeof_e

tmpl_fireballs_v::
    .db #e_type_enemy                                       ; type
    .db #(e_cmp_render | e_cmp_collider | e_cmp_physics | e_cmp_animated | e_cmp_ai)    ; cmps
    .db #0, #0                                              ; x, y
    .db #SPRT_FIREBALLS_V_W, #SPRT_FIREBALLS_V_H            ; w, h
    .db #-1, #0                                             ; vx, vy
    .dw #_sprt_fireballs_v_0                                ; sprite of the entity
    .db #0                                                  ; collides_against
    .db #0                                                  ; e_colliding_counter
    .db #2                                                  ; Lifes of the entity
    .dw #_fireballs_v_enemy                                 ; Animations
    .db #fireball_anim_counter                              ; Anim Counter
    .db #0                                                  ; Anim Status
    .dw #0                                                              ; ai function
    .db #0                                                              ; ai counter
    .dw #0                                                              ; prev pointer
CheckSize tmpl_fireballs_v, sizeof_e




;;======================================================================================================
;; Level Message 
;;

MESSAGE_TIME = level_message_anim_counter * 13

.macro defineMessageEntity posX, sprW, sprH, sprt, animation, anim_time
    .db #e_type_enemy                                                   ; type
    .db #(e_cmp_render |  e_cmp_physics | e_cmp_animated)               ; cmps
    .db #posX, #50                                                      ; x, y
    .db #sprW, #sprH                                                    ; w, h
    .db #0, #0                                                          ; vx, vy
    .dw #sprt                                                           ; sprite of the entity
    .db #0                                                              ; collides_against
    .db #0                                                              ; e_colliding_counter
    .db #0                                                              ; Lifes of the entity
    .dw #animation                                                      ; Animations
    .db #anim_time                                                      ; Anim Counter
    .db #0                                                              ; Anim Status
    .dw #sys_ai_behaviour_kill_level_message                                  ; ai function
    .db #MESSAGE_TIME                                                   ; ai counter
    .dw #0                                                              ; prev pointer
.endm

.macro defineLevelEntity posX, sprW, sprH, sprt, animation
    defineMessageEntity posX, sprW, sprH, sprt, animation, level_message_anim_counter
.endm

.macro defineNumberEntity posX, sprW, sprH, sprt, animation
    defineMessageEntity posX, sprW, sprH, sprt, animation, level_message_anim_counter-1
.endm




tmpl_message_level_passed_entity::
    defineLevelEntity #30, #SPR_LEVEL_MESSAGE_W, #SPR_LEVEL_MESSAGE_H, #_spr_level_message_0, #level_message_anim
CheckSize tmpl_message_level_passed_entity, sizeof_e

tmpl_message_level_1_number_entity::
    defineNumberEntity 46, SPR_NUMBER_MESSAGE_W, SPR_NUMBER_MESSAGE_H, _spr_number1_message_0, number1_message_anim
CheckSize tmpl_message_level_1_number_entity, sizeof_e

tmpl_message_level_2_number_entity::
    defineNumberEntity 46, SPR_NUMBER_MESSAGE_W, SPR_NUMBER_MESSAGE_H, _spr_number2_message_0, number2_message_anim
CheckSize tmpl_message_level_2_number_entity, sizeof_e

tmpl_message_level_3_number_entity::
    defineNumberEntity 46, SPR_NUMBER_MESSAGE_W, SPR_NUMBER_MESSAGE_H, _spr_number3_message_0, number3_message_anim
CheckSize tmpl_message_level_3_number_entity, sizeof_e

tmpl_message_level_4_number_entity::
    defineNumberEntity 46, SPR_NUMBER_MESSAGE_W, SPR_NUMBER_MESSAGE_H, _spr_number4_message_0, number4_message_anim
CheckSize tmpl_message_level_4_number_entity, sizeof_e

tmpl_message_level_5_number_entity::
    defineNumberEntity 46, SPR_NUMBER_MESSAGE_W, SPR_NUMBER_MESSAGE_H, _spr_number5_message_0, number5_message_anim
CheckSize tmpl_message_level_5_number_entity, sizeof_e


tmpl_you_wont_escape_entity::
    defineNumberEntity 30, SPR_YOU_WONT_ESCAPE_W, SPR_YOU_WONT_ESCAPE_H, _spr_you_wont_escape_message_0, you_wont_escape_message_anim
CheckSize tmpl_you_wont_escape_entity, sizeof_e


tmpl_nooo_entity::
    defineNumberEntity 30, SPR_NOOO_W, SPR_NOOO_H, _spr_nooo_message_0, nooo_message_anim
CheckSize tmpl_nooo_entity, sizeof_e


tmpl_invisible_entity::
    .db #e_type_dead                                                 ; type
    .db #0                                                              ; cmps
    .db #0, #0                                                          ; x, y
    .db #0, #0                                                          ; w, h
    .db #0, #0                                                          ; vx, vy
    .dw #0                                                              ; sprite of the entity
    .db #0                                                              ; collides_against
    .db #0                                                              ; e_colliding_counter
    .db #0                                                              ; Lifes of the entity
    .dw #0                                                              ; Animations
    .db #0                                                              ; Anim Counter
    .db #0                                                              ; Anim Status
    .dw #0                                                              ; ai function
    .db #0                                                              ; ai counter 
    .dw #0                                                              ; prev pointer
CheckSize tmpl_invisible_entity, sizeof_e
                                                            ; ai counter 
tmpl_red_planet::
    .db #e_type_alive                                                   ; type
    .db #(e_cmp_render | e_cmp_physics)                                 ; cmps
    .db #79 - SPRT_PLANET_W, #0                                         ; x, y
    .db #SPRT_PLANET_W, #SPRT_PLANET_H                                  ; w, h
    .db #0, #0                                                          ; vx, vy
    .dw #_sprt_planets_0                                                ; sprite of the entity
    .db #0                                                              ; collides_against
    .db #0                                                              ; e_colliding_counter
    .db #0                                                              ; Lifes of the entity
    .dw #0                                                              ; Animations
    .db #0                                                              ; Anim Counter
    .db #0                                                              ; Anim Status
    .dw #0                                                              ; ai function
    .db #0                                                              ; ai counter

tmpl_extra_life::
    .db #e_type_enemy                                       ; type
    .db #(e_cmp_render | e_cmp_collider | e_cmp_physics | e_cmp_animated | e_cmp_ai | e_cmp_extra_life)    ; cmps
    .db #0, #0                                              ; x, y
    .db #SPR_HEARTS_W, #SPR_HEARTS_H                        ; w, h
    .db #-1, #0                                             ; vx, vy
    .dw #_spr_hearts_0                                      ; sprite of the entity
    .db #0                                                  ; collides_against
    .db #0                                                  ; e_colliding_counter
    .db #0                                                  ; Lifes of the entity
    .dw #0                                                  ; Animations
    .db #0                                                  ; Anim Counter
    .db #0                                                  ; Anim Status
    .dw #0                                                  ; ai function
    .db #0                                                  ; ai counter
    .dw #0                                                              ; prev pointer
CheckSize tmpl_extra_life, sizeof_e
tmpl_blue_planet::
    .db #e_type_alive                                                   ; type
    .db #(e_cmp_render | e_cmp_physics)                                 ; cmps
    .db #79 - SPRT_PLANET_W, #0                                         ; x, y
    .db #SPRT_PLANET_W, #SPRT_PLANET_H                                  ; w, h
    .db #0, #0                                                          ; vx, vy
    .dw #_sprt_planets_1                                                ; sprite of the entity
    .db #0                                                              ; collides_against
    .db #0                                                              ; e_colliding_counter
    .db #0                                                              ; Lifes of the entity
    .dw #0                                                              ; Animations
    .db #0                                                              ; Anim Counter
    .db #0                                                              ; Anim Status
    .dw #0                                                              ; ai function
    .db #0                                                              ; ai counter

tmpl_green_planet::
    .db #e_type_alive                                                   ; type
    .db #(e_cmp_render | e_cmp_physics)                                 ; cmps
    .db #79 - SPRT_PLANET_W, #0                                         ; x, y
    .db #SPRT_PLANET_W, #SPRT_PLANET_H                                  ; w, h
    .db #0, #0                                                          ; vx, vy
    .dw #_sprt_planets_2                                                ; sprite of the entity
    .db #0                                                              ; collides_against
    .db #0                                                              ; e_colliding_counter
    .db #0                                                              ; Lifes of the entity
    .dw #0                                                              ; Animations
    .db #0                                                              ; Anim Counter
    .db #0                                                              ; Anim Status
    .dw #0                                                              ; ai function
    .db #0                                                              ; ai counter 