; MIT License

; Copyright (C) 2022 Z80Enjoyers / Luis Jesús Marhuenda Tendero (@ljmt1) / Alberto Vegara Belmonte (@avb52-ua) / Samuel Benjamín Oliva Bulpitt (@samuel0122)

; Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal 
; in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
; copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

; The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS 
; FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
; WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

;;======================================================================================================
.module Game_Manager
;;======================================================================================================
.area _CODE
;;======================================================================================================
.include "cpctelera.h.s"
.include "cpct_functions.inc"

.include "assets/music.inc"

.include "man/entity.h.s"
.include "man/levels.h.s"
.include "man/entity_templates.h.s"
.include "man/levels.h.s"
.include "man/macros.h.s"
.include "man/score.h.s"
.include "man/menu.h.s"

.include "sys/physics.h.s"
.include "sys/render.h.s"
.include "sys/input.h.s"
.include "sys/generator.h.s"
.include "sys/collision.h.s"
.include "sys/animations.h.s"
.include "sys/ai.h.s"

dead_counter::
    .db 0

;======================================================================================================
;   Manager Game Core Loop
;
;   Inputs:
;       - 
;   Outputs:
;       -
;
man_game_core_loop::
    ; (2B DE) songdata	Pointer to the start of the array containing song’s data in AKS binary format
    ld de, #_msc_menu
    call cpct_akp_musicInit_asm

    call man_menu_load_menu_screen
    call man_menu_options_keyboard

    ret

;======================================================================================================
;  Manager Game Init
;
man_game_init:
    call man_levels_init

    call man_entity_erase_all
    call sys_render_tilemap
    call sys_render_tilemap_planets

    call sys_generator_init

    ;; Creates player
    call man_entity_create
    
    ; DE has the position of the entity created
    ld      hl, #tmpl_player    ; FROM
    ld      bc, #sizeof_e       ; SIZE
    call cpct_memcpy_asm


    call sys_render_update
    call sys_render_show_lifes
    call cpct_waitVSYNC_asm

    ld a, #0
    ld (dead_counter), a
    ret

;======================================================================================================
;  Manager Game Restart level
;
man_game_restart:
    call man_levels_restart

    call man_entity_erase_all
    call sys_render_tilemap
    call sys_render_tilemap_planets
    
    call sys_generator_init

    ;; Creates player
    call man_entity_create
    
    ; DE has the position of the entity created
    ld      hl, #tmpl_player    ; FROM
    ld      bc, #sizeof_e       ; SIZE
    call cpct_memcpy_asm


    call sys_render_update
    call sys_render_show_lifes
    call cpct_waitVSYNC_asm

    ret

;======================================================================================================
;  Manager Game Player's win animation
;
man_game_win_animation:

    do_win_animation_loop:
    
        call sys_input_player_go_down_and_right
        ;; Updates player's position
        call sys_physics_update
        
        ;; Updates player's animation
        call sys_animations_update
        
        call cpct_waitVSYNC_asm

        

        ;; Kills players if its on right bound
        call sys_physics_kill_player_if_right_4animation

        ;; Renders the player
        call sys_render_update

        call man_entity_update

        call cpct_waitVSYNCStart_asm
        call cpct_waitVSYNC_asm

        ;; If player is dead, end loop
        ld ix, #entities
        ld a, e_type(ix)
        or a

        jr NZ, do_win_animation_loop

    ret


;======================================================================================================
;  Manager Game Play
;
man_game_play::
    loop:
        call cpct_waitVSYNC_asm
        call cpct_akp_musicPlay_asm

        call man_levels_update

        ; call sys_render_tilemap_planets

        call sys_input_update
        ; call sys_generator_generate
        call sys_generator_update

        call sys_ai_update

        ; call cpct_waitVSYNC_asm
        ; call sys_render_update_palette

        call sys_collision_update   ;; Check if this should be place here, or before sys_input
        call man_score_print_lifes_and_distance
        call man_entity_decrement_collding_counter

        ;; Check if the player has been killed, and stop the game
        IsPlayerDead _end_game_loop


        ;call sys_render_update_clear
        
        call sys_physics_update
        
        call sys_animations_update
        
        call cpct_waitVSYNC_asm
        call sys_render_update

        call sys_render_update_palette

        call man_entity_update

        call cpct_waitVSYNCStart_asm
        call cpct_waitVSYNC_asm

        ld  a, (are_all_levels_completed)
        or  a
        jr  Z,  loop

    _end_game_loop:


    ret



;======================================================================================================
;  Starts a new game
;
start_game::
    ; (2B DE) songdata	Pointer to the start of the array containing song’s data in AKS binary format
    ld de, #_msc_game
    call cpct_akp_musicInit_asm

    call man_score_reset_score

    call man_game_init

    jump_to_play:
    call man_game_play  
    
    ;; Play will only finish if all levels are completed or if player is dead
    ld  a, (are_all_levels_completed)
    or  a

    ;;;;;;;;;;;;;;;;;;;;;;;;
    ;; Case Player dead
    jr NZ, game_ended_player_won

        ;; Increment dead counter
        ; ld a, (dead_counter)
        ; inc a
        ; ld (dead_counter), a

        ld hl, (dead_counter)

        ld a, l
        add #0x01
        daa
        ld l, a

        ld (dead_counter), hl

        ;; Show dead screen with option to restart level or go back to main menu
        call man_menu_dead_screen

        ;; Checks A (0 -> back to main menu) (1 -> restart)
        or a

        ;; If A == 0 -> back to main menu
        jr Z, go_back_to_main_menu_start_game 

        ;; If restart 
        call man_game_restart
        jr jump_to_play

    ;;;;;;;;;;;;;;;;;;;;;;;;
    ;; Case Player Won
    game_ended_player_won:
        call cpct_akp_stop_asm
        ;; Show win screen with a press key to go back to main menu
        call man_game_win_animation
        call man_menu_win_screen

    go_back_to_main_menu_start_game:

    ;; After key is pressed, load menu and go to key bucle
    call sys_render_init
    call man_menu_load_menu_screen
    jp man_menu_options_keyboard

