; MIT License

; Copyright (C) 2022 Z80Enjoyers / Luis Jesús Marhuenda Tendero (@ljmt1) / Alberto Vegara Belmonte (@avb52-ua) / Samuel Benjamín Oliva Bulpitt (@samuel0122)

; Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal 
; in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
; copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

; The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS 
; FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
; WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

;;======================================================================================================
.module Levels_Manager
;;======================================================================================================
.area _CODE
;;======================================================================================================
.include "cpctelera.h.s"
.include "cpct_functions.inc"
.include "man/entity.h.s"
.include "sys/generator.h.s"
.include "sys/input.h.s"
.include "man/macros.h.s"
;;======================================================================================================

;;======================================================================================================
;;  Private Values
;;
current_level::
    .dw #levels-2

; Different level's combinations
levels:
    .dw #level_1_message_combination
    .dw #generable_combination_1
    .dw #level_2_message_combination
    .dw #generable_combination_2
    .dw #level_3_message_combination
    .dw #generable_combination_3
    .dw #level_4_message_combination
    .dw generable_combination_4
    .dw #level_5_message_combination
    .dw generable_combination_5

    .dw 0



level_has_been_passed:
    .db 0   ;; 0 if the level has not been passed, 1 if the leves has been passed


are_all_levels_completed::
    .db 0
;======================================================================================================
;  Initialices levels
;
;  Inputs:
;      - 
;
man_levels_init::  
    ld  hl, #levels-2
    ld  (current_level), hl

    ld  a, #1
    ld  (level_has_been_passed), a

    ld  a, #0
    ld  (are_all_levels_completed), a

    ret

;======================================================================================================
;  Restarts levels
;
;  Inputs:
;      - 
;
man_levels_restart::  
    ;; Go back "4" levels
    ld  hl, (current_level)
    dec hl
    dec hl
    dec hl
    dec hl
    ld  (current_level), hl


    ld  a, #1
    ld  (level_has_been_passed), a

    ld  a, #0
    ld  (are_all_levels_completed), a

    ret

;======================================================================================================
;  Updates levels
;
;  Inputs:
;      - 
;
man_levels_update::
    
    ;; Check if the current level has been passed
    ld  a, (level_has_been_passed)
    or  a
    jr NZ, try_start_a_new_level_levels_update  ;; If the level is finished, try to start the next level

        ;; If the level isn't finished, check if there's a combination in process by the generator
        ld  a, (theres_combination_in_process)
        or  a
        ret NZ  ;; If there's a combination in process, don't do anything

        ;; If there's no combination in process, check if there's only 1 entity alive (player) to start next level
        call man_entity_freeSpace
        cp #max_entities - 1
        ret NZ  ;; If there's more than 1 entity, don't generate new level yet

        ld  a, #1
        ld  (level_has_been_passed), a

    try_start_a_new_level_levels_update:

    ;; If its ended (no more levels to generate), ret
    ld  ix, (current_level)
    ld  de, #2
    add ix, de      
    
    ld  a, (ix)    ;; Cheks if pointer of next_level isn't null (ENDED)
    or 1(ix)
    jr Z, all_levels_are_completed

    ;; If there's no current level and it's not the end. Start next level
    ld  (current_level), ix

    ld  a, #0
    ld  (level_has_been_passed), a
    
    ;; Tell the generator there's a new level
    call sys_generator_start_new_combination

    ret

    all_levels_are_completed:
    ;; If all levels are completed, set it
    ld  a, #1
    ld  (are_all_levels_completed), a
    ret