; MIT License

; Copyright (C) 2022 Z80Enjoyers / Luis Jesús Marhuenda Tendero (@ljmt1) / Alberto Vegara Belmonte (@avb52-ua) / Samuel Benjamín Oliva Bulpitt (@samuel0122)

; Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal 
; in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
; copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

; The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS 
; FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
; WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

;;======================================================================================================
.module Macros_Manager
;;======================================================================================================
.area _CODE
;;======================================================================================================

;;======================================================================================================
;; Entity Templates Manager Macros
;;

;;------------------------------------------------------------------------------------------------------
;;  Check Size
;;
;;  Input:
;;      - __tmple_name:
;;      - __entity_size:
;;
.macro CheckSize __tmple_name, __entity_size
    this_tmpl_size = . - __tmple_name
    ; FILLS WITH 0
    ; .ds this_tmpl_size - __entity_size
    .ifne (this_tmpl_size - __entity_size)
        .error "Invalid Template Size"
    .endif
.endm

;;======================================================================================================
;; Levels Manager Macros
;;

;;------------------------------------------------------------------------------------------------------
;;  Check bit
;;
;;  Input:
;;      - E:
;;      - Tag:
;;
.macro CheckBit E, bits, Tag
    ld a, (E)
    and bits
    jr Z, Tag
.endm

;;======================================================================================================
;; Entity Manager Macros
;;

;;------------------------------------------------------------------------------------------------------
;;  IsValid
;;
;;  Input:
;;      - E:
;;      - Tag:
;;
.macro IsValid E, Tag
    ld      a, e_type(E)          ; Load in register A, the value of the type of the entity
    cp      #e_type_invalid
    jr      z, Tag              ; saltar si la ultima operacion ha dado cero
.endm

;;------------------------------------------------------------------------------------------------------
;;  IsDead
;;
;;  Input:
;;      - E:
;;      - Tag:
;;
.macro IsDead E, Tag
    ld      a, e_type(E)          ; Load in register A, the value of the type of the entity
    and     #e_type_dead
    jr      z, Tag              ; saltar si la ultima operacion ha dado cero
.endm

.macro IsNotDead E, Tag
    ld      a, e_type(E)        ; Load in register A, the value of the type of the entity
    and     #e_type_dead
    jr      nz, Tag             ; saltar si la ultima operacion no ha dado cero
.endm

.macro IsPlayerDead Tag
    ld  ix, #entities           ;; The player always will be our first entity
    IsNotDead ix, Tag
.endm

;;------------------------------------------------------------------------------------------------------
;;  SignatureMatches
;;
;;  Input:
;;      - E:
;;      - S:
;;      - Tag:
;;
.macro SignatureMatches E, S, Tag
    ld      a, e_cmps(E)
    and     S
    cp      S
    jr      z, Tag
.endm

;;------------------------------------------------------------------------------------------------------
;;  SignatureDoesNotMatch
;;
;;  Input:
;;      - E:
;;      - S:
;;      - Tag
;;
.macro SignatureDoesNotMatch E, S, Tag
    ld      a, e_cmps(E)
    and     S
    cp      S
    jr      nz, Tag
.endm

;;------------------------------------------------------------------------------------------------------
;;  NextEntity
;;
;;  Input:
;;      - E:
;;
.macro NextEntity E
    ld      de, #sizeof_e   ; e++;
    add     E, de           ; e++;
.endm

;;------------------------------------------------------------------------------------------------------
;;  SaveRegisters
;;
;;  Input:
;;      -
;;
.macro SaveRegisters
    push ix     ;; Entity
    push bc     ;; b: Signature
    push hl     ;; Function to be called
.endm

;;------------------------------------------------------------------------------------------------------
;;  RecoverRegisters
;;
;;  Input:
;;      -
;;
.macro RecoverRegisters
    pop hl
    pop bc
    pop ix
.endm

;;======================================================================================================
;; Physics System Macros
;;

;;------------------------------------------------------------------------------------------------------
;;  CheckLeftBound
;;
;;  Input:
;;      - E
;;      - Tag
;;
.macro CheckLeftBound E, Tag
    ;; Check if after moving, its out of right bound
    ld      a, e_x(E)
    ld      b, e_vx(E)
    add     a, b
    and     #0x80
    ;bit     7, a
    jr      Z, Tag      ;; If its not at left bound, jump
.endm

;;------------------------------------------------------------------------------------------------------
;;  CheckRightBound
;;
;;  Input:
;;      - E
;;      - Tag
;;
.macro CheckRightBound E, Tag
    ;; Check if after moving, its out of right bound
    ld      a, #SCREEN_WITDH        ;; /
    sub     e_w(E)                  ;;| C = right_bound = 80 - e->w;
    ld      c, a                    ;; \

    ld      b, e_x(E)               ;;  /
    ld      a, e_vx(E)              ;; | B = new_position = X + VX
    add     b                       ;;  \

    cp      c
    jr      c, Tag
.endm

;;------------------------------------------------------------------------------------------------------
;;  CheckTopBound
;;
;;  Input:
;;      - E
;;      - Tag
;;
.macro CheckTopBound E, Tag
    ld      a, e_y(E)               ;;  /
    add     a, e_vy(E)              ;; |    if v+vy > top_bound
    cp      #SCREEN_TOP_BOUND

    jp      NC, Tag

    ; ld      a, e_y(E)               ;;  /
    ; cp      #0                      ;; |    if y==top_bound
    ; jr      z, is_at_top_bound      ;;  \
; 
    ; and     #0x80                   ;; If position is high, can't be on top bound
    ; jr      nz, Tag
; 
    ; ld      a, e_y(E)               ;;  /
    ; add     a, e_vy(E)              ;; |    if v+vy > top_bound
    ; ld      b, a
    ; ld      a, #SCREEN_BOTTOM_BOUND
    ; cp      a, b
    ; ;and     #0x80
    ; jr      NC, Tag                  ;; \   

    is_at_top_bound:             ;; if v+vy < top_bound -> top_bound
.endm


;;------------------------------------------------------------------------------------------------------
;;  CheckTopBound ONLY when this top bound is limited to 0 (no limitations)
;;
;;  Input:
;;      - E
;;      - Tag
;;
.macro CheckTopBoundWhenIs0 E, Tag
    ld      a, e_y(E)               ;;  /
    cp      #0                      ;; |    if y==top_bound
    jr      z, is_at_top_bound      ;;  \

    and     #0x80                   ;; If position is high, can't be on top bound
    jr      nz, Tag

    ld      a, e_y(E)               ;;  /
    add     a, e_vy(E)              ;; |    if v+vy > top_bound
    ld      b, a
    ld      a, #SCREEN_BOTTOM_BOUND
    cp      a, b
    ;and     #0x80
    jr      NC, Tag                  ;; \   

    is_at_top_bound:             ;; if v+vy < top_bound -> top_bound
.endm

;;------------------------------------------------------------------------------------------------------
;;  CheckBottomBound
;;
;;  Input:
;;      - E
;;      - Tag
;;
.macro CheckBottomBound E, Tag
    ld      a, #SCREEN_BOTTOM_BOUND   ;; / 
    sub     e_h(E)              ;;| bottom_bound = 200 - e_entity->w;
    ld      b, a                ;; \

    ld      a, e_y(E)               ;;  /
    cp      b                       ;; |    if y==bottom_bound
    jr      z, is_at_bottom_bound   ;;  \

    ld      a, e_y(E)               ;;  /
    add     a, e_vy(E)              ;; |    compare(y+vy, bottom_bound)
    cp      b                       ;;  \
    jr      z, is_at_bottom_bound   ;; if y+vy == bottom_bound
    jr      c, Tag                  ;; if v+vy < vottom_bound

    is_at_bottom_bound:             ;; if v+vy > bottom_bound -> isBottom_bound
.endm

;;------------------------------------------------------------------------------------------------------
;;  CheckVxNegative
;;
;;  Input:
;;      - E
;;      - Tag
;;
.macro CheckVxNegative E, Tag
    ld      a, e_vx(E)
    and     #0x80       ;; Check negative bit
    jr      z, Tag
.endm

;;------------------------------------------------------------------------------------------------------
;;  CheckVxPositive
;;
;;  Input:
;;      - E
;;      - Tag
;;
.macro CheckVxPositive E, Tag
    ld      a, e_vx(E)
    and     #0x80       ;; Check negative bit
    jr      nz, Tag
.endm

;;------------------------------------------------------------------------------------------------------
;;  CheckVyNegative
;;
;;  Input:
;;      - E
;;      - Tag
;;
.macro CheckVyNegative E, Tag
    ld      a, e_vy(E)
    and     #0x80       ;; Check negative bit
    jr      z, Tag
.endm

;;------------------------------------------------------------------------------------------------------
;;  CheckVyPositive
;;
;;  Input:
;;      - E
;;      - Tag
;;
.macro CheckVyPositive E, Tag
    ld      a, e_vy(E)
    and     #0x80       ;; Check negative bit
    jr      nz, Tag
.endm

;;------------------------------------------------------------------------------------------------------
;;  CheckVyNot0
;;
;;  Input:
;;      - E
;;      - Tag
;;
.macro CheckVyNot0 E, Tag
    ld      a, e_vy(E)
    or      a           ;; Check if 0
    jr      z, Tag
.endm

;;======================================================================================================
;; Collision System Macros
;;

;;------------------------------------------------------------------------------------------------------
;;  CollideFromAbove
;;
;;  Input:
;;      - Left_e
;;      - Right_e
;;      - Tag
;;
COLLISION_MARGIN = 4
.macro CollideFromAbove Left_e, Right_e, Tag
    ld a, e_y (Right_e)
    
    add a, e_h (Right_e)
    sub #COLLISION_MARGIN
    
    ld b, e_y (Left_e)

    sub b

    jp C, Tag
.endm

;;------------------------------------------------------------------------------------------------------
;;  CollideFromBelow
;;
;;  Input:
;;      - Left_e
;;      - Right_e
;;      - Tag
;;
.macro CollideFromBelow Left_e, Right_e, Tag
    ld a, e_y (Left_e)

    add a, e_h (Left_e)
    sub #COLLISION_MARGIN

    ld b, e_y (Right_e)

    sub b

    jp C, Tag
.endm

;;------------------------------------------------------------------------------------------------------
;;  CollideAtEnemyLeftHand
;;
;;  Input:
;;      - Left_e
;;      - Right_e
;;      - Tag
;;
.macro CollideAtEnemyLeftHand Left_e, Right_e, Tag
    ld a, e_x (Right_e)

    add a, e_w (Right_e)
    sub #COLLISION_MARGIN

    ld b, e_x (Left_e)

    sub b

    jp C, Tag
.endm

;;------------------------------------------------------------------------------------------------------
;;  CollideAtEnemyRightHand
;;
;;  Input:
;;      - Left_e
;;      - Right_e
;;      - Tag
;;
.macro CollideAtEnemyRightHand Left_e, Right_e, Tag
    ld a, e_x (Left_e)

    add a, e_w (Left_e)
    sub #COLLISION_MARGIN

    ld b, e_x (Right_e)

    sub b

    jp C, Tag              
.endm


;;------------------------------------------------------------------------------------------------------
;;  IsCounterZero
;;
;;  Input:
;;      - E
;;      - Tag
;;
.macro IsCounterZero E, Tag

    ; ld  a, e_colliding_counter(E) ; 1
    ; or a
    ; jr NZ, Tag
    
    ;; Comprueba que sea menos que 1 (0)
    ld  a, e_colliding_counter(E) ; 1
    cp  #1
    jr C, Tag
    
    ; ld  a, e_colliding_counter(ix)
    ; cp  #0
    ; jr  z, Tag          ; In case of the counter is already zero should not be decremented
    ;     add   #-1
    ;     cp    #0
    ;     jr    z, Tag    ; If counter is not zero decrement and check if it zero
.endm

.macro IsCounterNoZero E, Tag
    ; ld  a, e_colliding_counter(E) ; 1
    ; or a
    ; jr Z, Tag

    ;; Comprueba si el counter es menor que 1 (0)
    ld  a, e_colliding_counter(E) ; 1
    cp  #1
    jr NC, Tag


    ; jr  nz, check_counter_minus_one_no_zero     ; In case of the counter is already zero should not be decremented
    ;     jr _end_aux_no_zero
    ; check_counter_minus_one_no_zero:
    ;      add   #-1
    ;      cp    #0
    ;      jr    nz, Tag                   ; If counter is not zero decrement and check if it zero
    ; 
    ; _end_aux_no_zero:
.endm

.macro PrintSolidBox color ; TODO: Remove this macro when sys_collision is finished
    ld  de, #CPCT_VMEM_START_ASM
    ld  a, color
    ld  c, e_w(ix)
    ld  b, e_h(ix)
    call cpct_drawSolidBox_asm
.endm