; MIT License

; Copyright (C) 2022 Z80Enjoyers / Luis Jesús Marhuenda Tendero (@ljmt1) / Alberto Vegara Belmonte (@avb52-ua) / Samuel Benjamín Oliva Bulpitt (@samuel0122)

; Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal 
; in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
; copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

; The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS 
; FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
; WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

;;======================================================================================================
.module Menu_Manager
;;======================================================================================================
.area _CODE
;;======================================================================================================
.include "cpctelera.h.s"
.include "cpct_functions.inc"
.include "assets/screens.inc"
.include "assets/music.inc"
.include "man/game.h.s"
.include "sys/render.h.s"
;;======================================================================================================

win_string_congrats_message:
    .asciz "Yey, you escaped!";

win_string_press_message:
    .asciz "Press ESC";

WIN_RETURN_REY = Key_Esc

key_actions:
    .dw Key_1,      start_game      ; DW because are symbols of 4 bytes
    .dw Key_2,      controls_menu   ;
    .dw Key_3,      credits_menu    ;
    .dw Key_4,      exit
    .dw 0                           ; Exit value for the loop

escape_action:
    .dw Key_Esc,    man_game_core_loop
    .dw 0


dead_action:
    .dw Key_Esc
    .db #0
    .dw Key_1
    .db #1
    .dw 0

;======================================================================================================
;   Manager Menu Options Keyboard
;
;   Inputs:
;       - 
;   Outputs:
;       -
;
man_menu_options_keyboard::
    infine_loop_ok:

    ;; Check keyboard
    call    cpct_scanKeyboard_f_asm ;; No input

    call cpct_waitVSYNC_asm
    call cpct_akp_musicPlay_asm

    ;; Key-Action Check-Call loop
    ld      iy, #key_actions-4 ;; Start before the first key in order to make te increment at the begining of the loop

    loop_keys_ok:
        ld  bc, #4  ;; / IY += 4 (Next Key)
        add iy, bc  ;; \

        ld  l, 0(iy)    ;; / HL = Next key
        ld  h, 1(iy)    ;; \

        ;; Check if key==null TODO: make macro for this check
        ld  a, l                ;; / A = H | L
        or  h                   ;; \
        jr  z, end_loop_keys_ok    ;; If A=0, Key=null, ret

        ;; Check if key is pressed
        call cpct_isKeyPressed_asm  ;; Input-> HL(key)
        jr  z, loop_keys_ok

        ld      l, 2(iy)    ;; / HL = action function pointer
        ld      h, 3(iy)    ;; \
        jp      (hl)        ;; Jump to actino function
    
    end_loop_keys_ok:
        jr infine_loop_ok

    ret

;======================================================================================================
;   Manager Menu Load Screen
;
;   Inputs:
;       - 
;   Outputs:
;       -
;
man_menu_load_menu_screen::
    ; (2B DE) dest_end	Ending (latest) byte of the destination (decompressed) array
    ; (2B HL) source_end	Ending (latest) byte of the source (compressed) array
    ld de, #0xFFFF
    ld hl, #_compressed_screen_menu_end
    call cpct_zx7b_decrunch_s_asm

    ret

;======================================================================================================
;   Show Controls Screen
;
;   Inputs:
;       - 
;   Outputs:
;       -
;
controls_menu:
    ; (2B DE) songdata	Pointer to the start of the array containing song’s data in AKS binary format
    ld de, #_msc_menu
    call cpct_akp_musicInit_asm
    
    ; (2B DE) dest_end	Ending (latest) byte of the destination (decompressed) array
    ; (2B HL) source_end	Ending (latest) byte of the source (compressed) array
    ld de, #0xFFFF
    ld hl, #_compressed_screen_control_end
    call cpct_zx7b_decrunch_s_asm

    infine_loop_com:

    ;; Check keyboard
    call    cpct_scanKeyboard_f_asm ;; No input

    call cpct_waitVSYNC_asm
    call cpct_akp_musicPlay_asm

    ;; Key-Action Check-Call loop
    ld      iy, #escape_action-4 ;; Start before the first key in order to make te increment at the begining of the loop

    loop_keys_com:
        ld  bc, #4  ;; / IY += 4 (Next Key)
        add iy, bc  ;; \

        ld  l, 0(iy)    ;; / HL = Next key
        ld  h, 1(iy)    ;; \

        ;; Check if key==null TODO: make macro for this check
        ld  a, l                ;; / A = H | L
        or  h                   ;; \
        jr  z, end_loop_keys_com    ;; If A=0, Key=null, ret

        ;; Check if key is pressed
        call cpct_isKeyPressed_asm  ;; Input-> HL(key)
        jr  z, loop_keys_com

        ld      l, 2(iy)    ;; / HL = action function pointer
        ld      h, 3(iy)    ;; \
        jp      (hl)        ;; Jump to actino function
    
    end_loop_keys_com:
        jr infine_loop_com

    ; call man_menu_core_loop
    ret

;======================================================================================================
;   Show Credits Screen
;
;   Inputs:
;       - 
;   Outputs:
;       -
;
credits_menu:
    ; (2B DE) songdata	Pointer to the start of the array containing song’s data in AKS binary format
    ld de, #_msc_menu
    call cpct_akp_musicInit_asm

    ; (2B DE) dest_end	Ending (latest) byte of the destination (decompressed) array
    ; (2B HL) source_end	Ending (latest) byte of the source (compressed) array
    ld de, #0xFFFF
    ld hl, #_compressed_screen_credits_end
    call cpct_zx7b_decrunch_s_asm

    infine_loop_crm:

    ;; Check keyboard
    call    cpct_scanKeyboard_f_asm ;; No input

    call cpct_waitVSYNC_asm
    call cpct_akp_musicPlay_asm

    ;; Key-Action Check-Call loop
    ld      iy, #escape_action-4 ;; Start before the first key in order to make te increment at the begining of the loop

    loop_keys_crm:
        ld  bc, #4  ;; / IY += 4 (Next Key)
        add iy, bc  ;; \

        ld  l, 0(iy)    ;; / HL = Next key
        ld  h, 1(iy)    ;; \

        ;; Check if key==null TODO: make macro for this check
        ld  a, l                ;; / A = H | L
        or  h                   ;; \
        jr  z, end_loop_keys_crm    ;; If A=0, Key=null, ret

        ;; Check if key is pressed
        call cpct_isKeyPressed_asm  ;; Input-> HL(key)
        jr  z, loop_keys_crm

        ld      l, 2(iy)    ;; / HL = action function pointer
        ld      h, 3(iy)    ;; \
        jp      (hl)        ;; Jump to actino function
    
    end_loop_keys_crm:
        jr infine_loop_crm

    ; call man_menu_core_loop
    ret

;======================================================================================================
;   Show Win Screen
;
;   Inputs:
;       - 
;   Outputs:
;       -
;
man_menu_win_screen::
    ; call sys_render_draw_win_message

    ;; Write congrats msg
    ld   de, #CPCT_VMEM_START_ASM ;; DE = Pointer to start of the screen
    ld    b, #80                  ;; B = y coordinate (24 = 0x18)
    ld    c, #8                  ;; C = x coordinate (16 = 0x10)

    call cpct_getScreenPtr_asm    ;; Calculate video memory location and return it in HL

    ld iy, #win_string_congrats_message
    call cpct_drawStringM0_asm

    ;; Write press msg
    ld   de, #CPCT_VMEM_START_ASM ;; DE = Pointer to start of the screen
    ld    b, #120                  ;; B = y coordinate (24 = 0x18)
    ld    c, #24                  ;; C = x coordinate (16 = 0x10)

    call cpct_getScreenPtr_asm    ;; Calculate video memory location and return it in HL

    ld iy, #win_string_press_message
    call cpct_drawStringM0_asm
    

    ;; Loop until any key pressed
    win_screen_loop:
        call    cpct_scanKeyboard_f_asm
        ld hl,  #WIN_RETURN_REY
        call    cpct_isKeyPressed_asm  ;; Input-> HL(key)
        ret NZ
    
    jr win_screen_loop


;======================================================================================================
;   Show Dead Screen
;
;   Inputs:
;       - 
;   Outputs:
;       - A: 1 (repeat), 0 (back to main menu)
;
man_menu_dead_screen::
    call sys_render_draw_dead_message
    call    cpct_akp_stop_asm   ;; Stop music

    infine_loop_dead_screen:

    ;; Check keyboard
    call    cpct_scanKeyboard_f_asm ;; No input
    call    cpct_waitVSYNC_asm

    ;; Key-Action Check-Call loop
    ld      iy, #dead_action-3 ;; Start before the first key in order to make te increment at the begining of the loop

    loop_keys_dead_screen:
        ld  bc, #3  ;; / IY += 4 (Next Key)
        add iy, bc  ;; \

        ld  l, 0(iy)    ;; / HL = Next key
        ld  h, 1(iy)    ;; \

        ;; Check if key==null TODO: make macro for this check
        ld  a, l                ;; / A = H | L
        or  h                   ;; \
        jr  z, infine_loop_dead_screen    ;; If key==null, restart loop

        ;; Check if key is pressed
        call cpct_isKeyPressed_asm  
        jr  z, loop_keys_dead_screen    ;; If not pressed, check next key

        ld      a, 2(iy)    ;; Get next byte in A and ret
        ret
    

;======================================================================================================
;   Show Credits Screen
;
;   Inputs:
;       - 
;   Outputs:
;       -
;
exit:
    ret