; MIT License

; Copyright (C) 2022 Z80Enjoyers / Luis Jesús Marhuenda Tendero (@ljmt1) / Alberto Vegara Belmonte (@avb52-ua) / Samuel Benjamín Oliva Bulpitt (@samuel0122)

; Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal 
; in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
; copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

; The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS 
; FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
; WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

;;======================================================================================================
.module Animations_System
;;======================================================================================================
.area _CODE
;;======================================================================================================
.include "cpct_functions.inc"
.include "man/entity.h.s"
.include "man/animations.h.s"
;;======================================================================================================


;;======================================================================================================
;;  System Animations Update One Entity
;;
sys_animations_update_one_entity:
    ; TEMPORAL: Comprobar si la entidad es el player
    ld a, e_type(ix)
    cp #e_type_player
    jr nz, not_player

        call sys_animations_calculate_animation_status

        ; Comprobar si el anim_status es el mismo que antes
        ld a, e_anim_status(ix)
        cp b
        jr z, not_player

            ld e_anim_status(ix), b

            call man_animations_get_animPtr_HL
            ld   e_animations(ix), l
            ld   e_animations+1(ix), h

            ld a, (hl)
            ld e_anim_count(ix), a

            ; Guardo el Sprite de *nextAnim en *sprite
            inc hl

            ld a, (hl) ; Leo de *nextSprite y escribo en *sprite el primer byte
            ld e_sprite(ix), a

            inc hl
            ld a, (hl) ; Leo de *nextSprite y escribo en *sprite el segundo byte
            ld e_sprite+1(ix), a

    not_player:
    ; Decremento el counter y lo obtengo
    dec e_anim_count(ix)
    ld a, e_anim_count(ix)

    ; Si el counter no es 0, ret
    cp #0
    ret NZ

    ; Si el counter llegó a 0, pasamos a la siguiente animacion
    ld c, e_animations(ix)
    ld b, e_animations+1(ix) ; BC -> animation (puntero al animation actual)

    ; Obtenemos puntero al sig animation
    ld hl, #3
    add hl, bc ; HL -> *nextAnimation

    ; Compruebo si time de este anim es 0 o no
    ld a, (hl)  ; A -> *nextAnimation.time (nullptr)
    cp #0

    jp NZ, not_Zero_animationOneEntity ; Si no es 0, salto para coger el time y Sprite
        ; Si es 0 hay reiniciar la animación, pongo en *nextAnimation el puntero del inicio de la animación
        inc hl      ; HL -> *resetAnimation
        ld e, (hl)
        inc hl
        ld d, (hl)  ; DE -> *initAnimation

        ex de, hl   ; HL -> *initAnimation

        ld a, (hl)  ; A -> time

    not_Zero_animationOneEntity:
    ; HL -> *nextAnimation [(time, *sprite), (time, *sprite), ..., (0, initAnimation)]
    ; De -> *entity
    ; A  -> time

    ; Guardo animCounter, Sprite y el pointer anim
    ; Guardo counter
    ld e_anim_count(ix), a

    ; Guardo *nextAnimation
    ld e_animations(ix), l
    ld e_animations+1(ix), h ; HL -> animation (puntero al animation actual)

    ; Guardo el Sprite de *nextAnim en *sprite
    inc hl

    ld a, (hl) ; Leo de *nextSprite y escribo en *sprite el primer byte
    ld e_sprite(ix), a

    inc hl
    ld a, (hl) ; Leo de *nextSprite y escribo en *sprite el segundo byte
    ld e_sprite+1(ix), a

    ret

;;======================================================================================================
;;  System Animations Update
;;
;;
sys_animations_update::
    ld hl, #sys_animations_update_one_entity
    ld  b, #e_cmp_animated
    call man_entity_forall_matching
    ret

;;======================================================================================================
;;  System Animations Calculate Animation Status
;;  
;;  Input:
;;      -IX: Entity
;;  Output:
;;      -B: Animation Status
;;  Preconditions:
;;      0000 Standing
;;      0001 Left
;;      0010 Right
;;      0011 ---
;;      0100 Down
;;      0101 Down-Left
;;      0110 Down-Right
;;      0111 ---
;;      1000 Up
;;      1001 Up-Left
;;      1010 Up-Right
;;      1011 ---
;;
sys_animations_calculate_animation_status:
    ld b, #0 ;; B = Original status

    ld a, e_vx(ix)  ;; A = e_vx
    or a
    jr z, _vxzero

    _vxnozero:
        jp m, _vxnegative
    _vxpositive:
        set 1, b
        jr _vxzero
    _vxnegative:
        set 0, b
    _vxzero:

    ld a, e_vy(ix)
    or a
    jr z, _vyzero

    _vynozero:
        jp m, _vynegative
    _vypositive: ; Baja
        set 2, b
        jr _vyzero
    _vynegative: ; Sube
        set 3, b
    _vyzero:

    ;; Checks if player is at top bound
    ld a, e_y(ix)
    cp #SCREEN_TOP_BOUND
    ret NZ      ;; If not a top bound, ret

    ;; If at top bound, set flying
    set 3, b
    ret
