; MIT License

; Copyright (C) 2022 Z80Enjoyers / Luis Jesús Marhuenda Tendero (@ljmt1) / Alberto Vegara Belmonte (@avb52-ua) / Samuel Benjamín Oliva Bulpitt (@samuel0122)

; Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal 
; in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
; copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

; The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS 
; FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
; WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

;;======================================================================================================
.module Collision_System
;;======================================================================================================
.area _CODE
;;======================================================================================================
.include "man/entity.h.s"
.include "cpctelera.h.s"
.include "cpct_functions.inc"
.include "man/macros.h.s"
.include "sys/render.h.s"
;;======================================================================================================

COLLIDING_COUNTER = 30


;;======================================================================================================
;;  COLLISION UPDATE
;; 
;;  Outputs:
;;      - HL, POINTER TO THE SYS_PHYSICS_UPDATE_ONE_ENTITY
;;
sys_collision_update::
    ld    hl, #sys_collision_update_entities
    ld    b, #e_cmp_collider
    call  man_entity_forall_pairs_matching

    ret

;;======================================================================================================
;;  COLLISION UPDATE TWO ENTITIES
;; 
;;  Inputs:
;;      - IX, left entity
;;      - IY, right entity
;;
sys_collision_update_entities::
    ;; e1_collide_e2 = e1->collides_against & e2->type;
    ;; e2_collide_e1 = e2->collides_against & e1->type;
    ;; if ( (e1_collide_e2 | e2_collide_e1 ) == 0) return

    ld a, e_collides_against(ix)
    ld b, e_type(iy)
    and b

    cp #e_type_enemy
    ret NZ
        
        ;; Checks for collision
        call sys_collisions_entities_colliding

    ret

;;======================================================================================================
;;  Check if both entities are colliding, in that case the player must lose a life
;; 
;;  Inputs:
;;      - IX, left entity
;;      - IY, right entity
;;
sys_collisions_entities_colliding:

    ; PrintSolidBox #0x00 ; TODO: Remove this when sys_collision is finished

    CollideFromAbove ix, iy, end_entities_colliding

        CollideFromBelow ix, iy, end_entities_colliding

            CollideAtEnemyLeftHand ix, iy, end_entities_colliding

                CollideAtEnemyRightHand ix, iy, end_entities_colliding

                    IsCounterNoZero ix, end_entities_colliding                ; Implicit A = e_colliding_counter - 1
                        
                        SignatureDoesNotMatch iy, #e_cmp_extra_life, lose_life

                            call man_entity_increase_life

                            push iy ;; Save hearthPointer
                            pop  ix ;; Load hearthPointer and send it to destroy
                            call man_entity_set4destroy
                            
                            jr end_entities_colliding

                        lose_life:
                            ;; Reduce player left Life
                            call sys_render_print_invulnerability

                            call man_entity_lose_life

                            ld  e_colliding_counter(ix), #COLLIDING_COUNTER    ; Reset e_colliding_counter

                            
                            push ix ;; Save playerPointer
                            push iy ;; Save enemyPointer
                            pop  ix ;; Load enemyPointer and send it to destroy
                            call man_entity_set4destroy
                            
                            pop ix ;; Load playerPointer

                            ; PrintSolidBox #0xFF ; TODO: Remove this when sys_collision is finished

                            ;; If the player is killed the game should be ended
                            ;; so when the lifes of the player are zero
                            ;; the game ends
                            ld  a, e_lifes(ix)
                            or a
                            ret  NZ

                            ;; Si life = 0 -> dead
                            call man_entity_set4destroy

    end_entities_colliding:
    ret

    