; MIT License

; Copyright (C) 2022 Z80Enjoyers / Luis Jesús Marhuenda Tendero (@ljmt1) / Alberto Vegara Belmonte (@avb52-ua) / Samuel Benjamín Oliva Bulpitt (@samuel0122)

; Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal 
; in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
; copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

; The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS 
; FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
; WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

;;======================================================================================================
.module Generator_System
;;======================================================================================================
.area _CODE
;;======================================================================================================
.include "cpctelera.h.s"
.include "cpct_functions.inc"
.include "assets/sprites.inc"

.include "man/animations.h.s"
.include "man/entity.h.s"
.include "man/entity_templates.h.s"
.include "man/macros.h.s"
.include "man/levels.h.s"

.include "sys/ai.h.s"
;;======================================================================================================


;;======================================================================================================
;;  Private Values
;;


;;------------------------------------------------------------------------------------------------------
;; Combinations
;;

;; Consts
    INIT_X_SMALL_FIREBALL   = 80 - SPRT_SMALL_FIREBALL_W
    INIT_X_MEDIUM_FIREBALL  = 80 - SPRT_MEDIUM_FIREBALL_W
    INIT_X_BIG_FIREBALL     = 80 - SPRT_BIG_FIREBALL_W
    INIT_X_FIREBALLS_H      = 80 - SPRT_FIREBALLS_H_W
    INIT_X_FIREBALLS_V      = 80 - SPRT_FIREBALLS_V_W
    INIT_X_EXTRA_LIFE       = 80 - SPR_HEARTS_W
    INIT_X_FIREBALLS_H_LARGER      = 80 - SPRT_FIREBALLS_H_LARGER_0_W
    
    X_FOR_LEVEL_MESSAGE = 30
    X_FOR_NUMBER_MESSAGE = 48

    ;; How many possible combinations there are
    POSSIBLE_COMBINATIONS = 1
    ;; How many bytes does a single generation have
    SINGLE_GENERATE_SIZE = 7

    ;; Bytes of a single generation's meaning
    gen_comb_count      = 0
    gen_comb_entIdx     = 1
    gen_comb_entX       = 2
    gen_comb_entY       = 3
    gen_comb_aiCounter  = 4
    gen_comb_aiFunc     = 5

    MESSAGE_TIME = level_message_anim_counter * 13

;; Counter for the "line" of the combination
generator_counter:
    .db #0

;; Boolean to know if there's a combination generating in process (continue it) or not (start a new combination)
theres_combination_in_process::
    .db #0

;; Pointer to which combination I'm generating and where I am (which "line" in that combination)
combination_generating:
    .dw #0




;; List of avaliable templates
generable_entities:
    .dw #tmpl_small_fireball    ;; 0
    .dw #tmpl_medium_fireball   ;; 1
    .dw #tmpl_big_fireball      ;; 2
    .dw #tmpl_fireballs_h       ;; 3
    .dw #tmpl_fireballs_v       ;; 4

    .dw #tmpl_message_level_passed_entity   ;; 5
    .dw #tmpl_message_level_1_number_entity ;; 6
    .dw #tmpl_message_level_2_number_entity ;; 7
    .dw #tmpl_message_level_3_number_entity ;; 8
    .dw #tmpl_message_level_4_number_entity ;; 9
    .dw #tmpl_message_level_5_number_entity ;; 10

    .dw #tmpl_extra_life        ;; 11
    .dw #tmpl_fireballs_h_larger ;; 12

    .dw #tmpl_you_wont_escape_entity ;; 13
    .dw #tmpl_nooo_entity       ;; 14


;; The differents combinations

.macro DefineGenerableEntity Counter, GenerableEntity, X, Y, AICounter, AIBehaviour
    .db Counter
    .db GenerableEntity
    .db X
    .db Y
    .db AICounter
    .dw AIBehaviour
.endm
;; -----------------------------------------------------
;; Levels combinations
;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Levels messages
;;
    level_1_message_combination::
        ;;                  [ (Counter, generable_entity, x, y, ai_counter, ai_func ), (...), ... ,(0) ]
        DefineGenerableEntity #1  , #5, X_FOR_LEVEL_MESSAGE, #100, #MESSAGE_TIME, #sys_ai_behaviour_kill_level_message
        DefineGenerableEntity #1  , #6, X_FOR_NUMBER_MESSAGE, #100, #MESSAGE_TIME-1, #sys_ai_behaviour_kill_level_message

        .db #0

    level_2_message_combination::
        ;;                  [ (Counter, generable_entity, x, y, ai_counter, ai_func ), (...), ... ,(0) ]
        DefineGenerableEntity #1  , #5, X_FOR_LEVEL_MESSAGE, #100, #MESSAGE_TIME, #sys_ai_behaviour_kill_level_message
        DefineGenerableEntity #1  , #7, X_FOR_NUMBER_MESSAGE, #100, #MESSAGE_TIME-1, #sys_ai_behaviour_kill_level_message

        .db #0

    level_3_message_combination::
        ;;                  [ (Counter, generable_entity, x, y, ai_counter, ai_func ), (...), ... ,(0) ]
        DefineGenerableEntity #1  , #5, X_FOR_LEVEL_MESSAGE, #100, #MESSAGE_TIME, #sys_ai_behaviour_kill_level_message
        DefineGenerableEntity #1  , #8, X_FOR_NUMBER_MESSAGE, #100, #MESSAGE_TIME-1, #sys_ai_behaviour_kill_level_message

        .db #0

    level_4_message_combination::
        ;;                  [ (Counter, generable_entity, x, y, ai_counter, ai_func ), (...), ... ,(0) ]
        DefineGenerableEntity #1  , #5, X_FOR_LEVEL_MESSAGE, #100, #MESSAGE_TIME, #sys_ai_behaviour_kill_level_message
        DefineGenerableEntity #1  , #9, X_FOR_NUMBER_MESSAGE, #100, #MESSAGE_TIME-1, #sys_ai_behaviour_kill_level_message

        .db #0

    level_5_message_combination::
        ;;                  [ (Counter, generable_entity, x, y, ai_counter, ai_func ), (...), ... ,(0) ]
        DefineGenerableEntity #1  , #5, X_FOR_LEVEL_MESSAGE, #100, #MESSAGE_TIME, #sys_ai_behaviour_kill_level_message
        DefineGenerableEntity #1  , #10, X_FOR_NUMBER_MESSAGE, #100, #MESSAGE_TIME-1, #sys_ai_behaviour_kill_level_message

        .db #0


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Different levels
;;

;; Introductional level by Samuel
generable_combination_1::
    ;; [ (Counter, generable_entity, x, y, ai_counter, ai_func ), (...), ... ,(0) ]

    ;; Big run
    DefineGenerableEntity #20, #2, INIT_X_BIG_FIREBALL, #(SCREEN_BOTTOM_BOUND-SCREEN_TOP_BOUND)/2+SCREEN_TOP_BOUND, #0, #sys_ai_behaviour_stop_and_run
    DefineGenerableEntity #20, #2, INIT_X_BIG_FIREBALL, #SCREEN_TOP_BOUND+20,                                       #0, #sys_ai_behaviour_stop_and_run
    DefineGenerableEntity #20, #2, INIT_X_BIG_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_BIG_FIREBALL_H,                   #0, #sys_ai_behaviour_stop_and_run
   
    ;; Small up-down
    DefineGenerableEntity #100, #0, INIT_X_SMALL_FIREBALL, #SCREEN_TOP_BOUND+40 ,                                                                   #0, #sys_ai_behaviour_up_down
    DefineGenerableEntity #1  , #0, INIT_X_SMALL_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_SMALL_FIREBALL_H-20 ,                                          #0, #sys_ai_behaviour_up_down
   
    ;; Med up-down
    DefineGenerableEntity #125, #1, INIT_X_MEDIUM_FIREBALL, #SCREEN_TOP_BOUND ,                                                                     #0, #sys_ai_behaviour_up_down
    DefineGenerableEntity #1  , #1, INIT_X_MEDIUM_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_MEDIUM_FIREBALL_H ,                                           #0, #sys_ai_behaviour_up_down

    ;; Tripple small up-down
    DefineGenerableEntity #100, #0, INIT_X_SMALL_FIREBALL, #SCREEN_TOP_BOUND+30 ,                                                                   #0, #sys_ai_behaviour_up_down
    DefineGenerableEntity #1  , #0, INIT_X_SMALL_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_SMALL_FIREBALL_H-45 ,                                          #0, #sys_ai_behaviour_up_down
    DefineGenerableEntity #1  , #0, INIT_X_SMALL_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_SMALL_FIREBALL_H ,                                             #0, #sys_ai_behaviour_up_down

    ;; Big do-squares
    DefineGenerableEntity #100, #2, INIT_X_BIG_FIREBALL, #SCREEN_TOP_BOUND+20 ,                                                                     #0, #sys_ai_behaviour_do_squares
    DefineGenerableEntity #1  , #2, INIT_X_BIG_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_MEDIUM_FIREBALL_H-35 ,                                           #0, #sys_ai_behaviour_do_squares

    ;; Big cross
    DefineGenerableEntity #150                      , #2, INIT_X_BIG_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_MEDIUM_FIREBALL_H-5 ,                      #0, #sys_ai_behaviour_up
    DefineGenerableEntity #SPRT_BIG_FIREBALL_W      , #2, INIT_X_BIG_FIREBALL, #SCREEN_TOP_BOUND+5 ,                                                #0, #sys_ai_behaviour_down

    ;; Heart
    DefineGenerableEntity #50 , #11, INIT_X_EXTRA_LIFE,     #SCREEN_TOP_BOUND+30 ,                                                                  #0, #sys_ai_behaviour_straight_forward_difficult

    ;; Vertical up-down
    DefineGenerableEntity #50,  #4, INIT_X_FIREBALLS_V,  #SCREEN_TOP_BOUND+46 ,                                                                     #0, #sys_ai_behaviour_up_down
    DefineGenerableEntity #1  , #4, INIT_X_FIREBALLS_V,  #SCREEN_BOTTOM_BOUND-SPRT_FIREBALLS_V_H ,                                                  #0, #sys_ai_behaviour_up_down

    ;; Vertical&horitzontal do-squares
    DefineGenerableEntity #150  , #4, INIT_X_FIREBALLS_V,  #SCREEN_TOP_BOUND ,                                                                          #0, #sys_ai_behaviour_do_squares
    DefineGenerableEntity #1    , #3, INIT_X_FIREBALLS_H,  #SCREEN_TOP_BOUND+SPRT_FIREBALLS_V_H ,                                                       #0, #sys_ai_behaviour_do_squares
    DefineGenerableEntity #20   , #4, INIT_X_FIREBALLS_V,  #SCREEN_BOTTOM_BOUND-SPRT_FIREBALLS_V_H-5 ,                                                  #0, #sys_ai_behaviour_stop_and_run

    .db #0


;; Created by samuel
generable_combination_2::
    ;; [ (Counter, generable_entity, x, y, ai_counter, ai_func ), (...), ... ,(0) ]
    ;; Up-down small
    DefineGenerableEntity #5, #0, INIT_X_SMALL_FIREBALL, #SCREEN_TOP_BOUND ,                                                                 #0, #sys_ai_behaviour_up_down
    DefineGenerableEntity #1, #0, INIT_X_SMALL_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_SMALL_FIREBALL_H ,                                        #0, #sys_ai_behaviour_up_down
    DefineGenerableEntity #1, #1, INIT_X_MEDIUM_FIREBALL, #(SCREEN_BOTTOM_BOUND-SCREEN_TOP_BOUND+SPRT_SMALL_FIREBALL_H)/2+SCREEN_TOP_BOUND , #0, #sys_ai_behaviour_straight_forward
    
    ;; Stop-run small
    DefineGenerableEntity #100, #0, INIT_X_SMALL_FIREBALL, #SCREEN_TOP_BOUND+40 ,                          #0, #sys_ai_behaviour_stop_and_run
    DefineGenerableEntity #1  , #0, INIT_X_SMALL_FIREBALL, #SCREEN_TOP_BOUND,                              #0, #sys_ai_behaviour_stop_and_run
    DefineGenerableEntity #1  , #0, INIT_X_SMALL_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_SMALL_FIREBALL_H ,    #0, #sys_ai_behaviour_stop_and_run
    DefineGenerableEntity #1  , #0, INIT_X_SMALL_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_SMALL_FIREBALL_H-40 , #0, #sys_ai_behaviour_stop_and_run

    ;; Squares med
    DefineGenerableEntity #100, #1, INIT_X_MEDIUM_FIREBALL, #SCREEN_TOP_BOUND ,                                                                 #0, #sys_ai_behaviour_do_squares
    DefineGenerableEntity #1,   #1, INIT_X_MEDIUM_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_MEDIUM_FIREBALL_H ,                                       #0, #sys_ai_behaviour_do_squares
    DefineGenerableEntity #1,   #0, INIT_X_SMALL_FIREBALL , #(SCREEN_BOTTOM_BOUND-SCREEN_TOP_BOUND+SPRT_SMALL_FIREBALL_H)/2+SCREEN_TOP_BOUND ,  #0, #sys_ai_behaviour_stop_and_run

    ;; Cross med
    DefineGenerableEntity #100,                     #1, INIT_X_MEDIUM_FIREBALL, #SCREEN_TOP_BOUND+20 ,                          #0, #sys_ai_behaviour_down
    DefineGenerableEntity #SPRT_MEDIUM_FIREBALL_W,  #1, INIT_X_MEDIUM_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_MEDIUM_FIREBALL_H ,   #0, #sys_ai_behaviour_up

    ;; Stop-run bigs with medium up-down
    DefineGenerableEntity #100, #2, INIT_X_BIG_FIREBALL,    #SCREEN_TOP_BOUND+12 ,                                          #0, #sys_ai_behaviour_stop_and_run
    DefineGenerableEntity #1  , #2, INIT_X_BIG_FIREBALL,    #SCREEN_BOTTOM_BOUND-SPRT_BIG_FIREBALL_H-12 ,                   #0, #sys_ai_behaviour_stop_and_run
    DefineGenerableEntity #1  , #1, INIT_X_MEDIUM_FIREBALL, #(SCREEN_BOTTOM_BOUND-SCREEN_TOP_BOUND+30)/2+SCREEN_TOP_BOUND , #0, #sys_ai_behaviour_up_down
    DefineGenerableEntity #10 , #11, INIT_X_EXTRA_LIFE,     #(SCREEN_BOTTOM_BOUND-SCREEN_TOP_BOUND+30)/2+SCREEN_TOP_BOUND , #0, #sys_ai_behaviour_straight_forward

    ;; Stop-run bigs with small squares
    DefineGenerableEntity #100, #2, INIT_X_BIG_FIREBALL,   #SCREEN_TOP_BOUND+10 ,                                                                   #0, #sys_ai_behaviour_stop_and_run
    DefineGenerableEntity #1  , #2, INIT_X_BIG_FIREBALL,   #SCREEN_BOTTOM_BOUND-SPRT_BIG_FIREBALL_H-10 ,                                            #0, #sys_ai_behaviour_stop_and_run
    DefineGenerableEntity #1  , #0, INIT_X_SMALL_FIREBALL, #(SCREEN_BOTTOM_BOUND-SCREEN_TOP_BOUND+SPRT_SMALL_FIREBALL_H-16)/2+SCREEN_TOP_BOUND ,    #0, #sys_ai_behaviour_do_squares

    ;; Up-down horitzontal
    DefineGenerableEntity #150, #3, INIT_X_FIREBALLS_H,     #SCREEN_TOP_BOUND+50 ,                              #0, #sys_ai_behaviour_up_down
    DefineGenerableEntity #1  , #3, INIT_X_FIREBALLS_H,     #SCREEN_BOTTOM_BOUND-SPRT_FIREBALLS_H_H-20 ,        #0, #sys_ai_behaviour_up_down
    DefineGenerableEntity #1  , #1, INIT_X_MEDIUM_FIREBALL, #SCREEN_TOP_BOUND+10 ,                              #0, #sys_ai_behaviour_straight_forward
    DefineGenerableEntity #1  , #1, INIT_X_MEDIUM_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_MEDIUM_FIREBALL_H-10 ,    #0, #sys_ai_behaviour_straight_forward

    ;; Stop-run bigs with vertical middle
    DefineGenerableEntity #100, #2, INIT_X_BIG_FIREBALL, #SCREEN_TOP_BOUND+10 ,                                                             #0, #sys_ai_behaviour_stop_and_run
    DefineGenerableEntity #1  , #2, INIT_X_BIG_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_BIG_FIREBALL_H-10 ,                                      #0, #sys_ai_behaviour_stop_and_run
    DefineGenerableEntity #1  , #4, INIT_X_FIREBALLS_V,  #(SCREEN_BOTTOM_BOUND-SCREEN_TOP_BOUND-SPRT_FIREBALLS_V_H+15)/2+SCREEN_TOP_BOUND , #0, #sys_ai_behaviour_up_down


    ;; Crazy
    DefineGenerableEntity #150,  #0, INIT_X_SMALL_FIREBALL, #SCREEN_BOTTOM_BOUND-10,    #0, #sys_ai_behaviour_do_squares
    DefineGenerableEntity #5  ,  #0, INIT_X_SMALL_FIREBALL, #80 ,                       #0, #sys_ai_behaviour_stop_and_run
    DefineGenerableEntity #5  ,  #0, INIT_X_SMALL_FIREBALL, #SCREEN_BOTTOM_BOUND-10 ,   #0, #sys_ai_behaviour_up_down

    DefineGenerableEntity #100, #0, INIT_X_SMALL_FIREBALL, #SCREEN_BOTTOM_BOUND-100 , #0, #sys_ai_behaviour_up_down
    DefineGenerableEntity #5  , #0, INIT_X_SMALL_FIREBALL, #SCREEN_BOTTOM_BOUND-20 ,  #0, #sys_ai_behaviour_stop_and_run

    DefineGenerableEntity #100,                 #2, INIT_X_BIG_FIREBALL, #SCREEN_TOP_BOUND ,                        #0, #sys_ai_behaviour_down
    DefineGenerableEntity #SPRT_BIG_FIREBALL_W, #2, INIT_X_BIG_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_BIG_FIREBALL_H , #0, #sys_ai_behaviour_up

    DefineGenerableEntity #100, #2, INIT_X_BIG_FIREBALL,    #SCREEN_TOP_BOUND ,                                                                 #0, #sys_ai_behaviour_up_down
    DefineGenerableEntity #1  , #0, INIT_X_SMALL_FIREBALL,  #(SCREEN_BOTTOM_BOUND-SCREEN_TOP_BOUND+SPRT_SMALL_FIREBALL_H)/2+SCREEN_TOP_BOUND,   #0, #sys_ai_behaviour_stop_and_run
    DefineGenerableEntity #1  , #2, INIT_X_BIG_FIREBALL,    #SCREEN_BOTTOM_BOUND-SPRT_BIG_FIREBALL_H ,                                          #0, #sys_ai_behaviour_up_down

    .db #0
    

;; Intensive level created by Samu
generable_combination_3::
    ;; [ (Counter, generable_entity, x, y, ai_counter, ai_func ), (...), ... ,(0) ]
    ;; Verticals to go through middle
    DefineGenerableEntity #50  , #4, INIT_X_FIREBALLS_V   , #SCREEN_TOP_BOUND+10,                           #0, #sys_ai_behaviour_straight_forward
    DefineGenerableEntity #1   , #4, INIT_X_FIREBALLS_V   , #SCREEN_BOTTOM_BOUND-SPRT_FIREBALLS_V_H-10,     #0, #sys_ai_behaviour_straight_forward

    ;; Verticals up-down to go through middle
    DefineGenerableEntity #75  , #4, INIT_X_FIREBALLS_V   , #SCREEN_TOP_BOUND+30,                           #1, #sys_ai_behaviour_up_down
    DefineGenerableEntity #1   , #4, INIT_X_FIREBALLS_V   , #SCREEN_BOTTOM_BOUND-SPRT_FIREBALLS_V_H,        #0, #sys_ai_behaviour_up_down
    
    ;; Corner H+V with med-run
    DefineGenerableEntity #100 , #3, INIT_X_FIREBALLS_H    , #SCREEN_TOP_BOUND,                             #0, #sys_ai_behaviour_do_squares
    DefineGenerableEntity #1   , #4, INIT_X_FIREBALLS_V    , #SCREEN_TOP_BOUND+60,                          #0, #sys_ai_behaviour_do_squares
    DefineGenerableEntity #1   , #1, INIT_X_MEDIUM_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_MEDIUM_FIREBALL_H,   #0, #sys_ai_behaviour_stop_and_run

    ;;Vertical+horitzontal up-down
    DefineGenerableEntity #150  , #3, INIT_X_FIREBALLS_H   , #SCREEN_BOTTOM_BOUND-SPRT_FIREBALLS_H_H-10,    #1, #sys_ai_behaviour_up_down
    DefineGenerableEntity #1    , #4, INIT_X_FIREBALLS_V   , #SCREEN_TOP_BOUND+30,                          #0, #sys_ai_behaviour_up_down
    
    ;; All meds run
    DefineGenerableEntity #200  , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_MEDIUM_FIREBALL_H,     #0, #sys_ai_behaviour_stop_and_run
    DefineGenerableEntity #20   , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_MEDIUM_FIREBALL_H-40,  #0, #sys_ai_behaviour_stop_and_run
    DefineGenerableEntity #20   , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_MEDIUM_FIREBALL_H-80,  #0, #sys_ai_behaviour_stop_and_run
    DefineGenerableEntity #20   , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_TOP_BOUND,                               #0, #sys_ai_behaviour_stop_and_run
    DefineGenerableEntity #10   , #11, INIT_X_EXTRA_LIFE     , #SCREEN_TOP_BOUND+20 ,                           #0, #sys_ai_behaviour_straight_forward_difficult
    
    .db #0


;; Maze created by alberto
generable_combination_4::
    ;; [ (Counter, generable_entity, x, y, ai_counter, ai_func ), (...), ... ,(0) ]    
    DefineGenerableEntity #5, #4, INIT_X_FIREBALLS_V, #40 , #0, #sys_ai_behaviour_straight_forward
    DefineGenerableEntity #1, #4, INIT_X_FIREBALLS_V, #120 , #0, #sys_ai_behaviour_straight_forward

    DefineGenerableEntity #27, #3, INIT_X_FIREBALLS_H, #70 , #0, #sys_ai_behaviour_straight_forward

    DefineGenerableEntity #10, #4, INIT_X_FIREBALLS_V, #70 , #0, #sys_ai_behaviour_straight_forward

    DefineGenerableEntity #27, #3, INIT_X_FIREBALLS_H, #120 , #0, #sys_ai_behaviour_straight_forward

    DefineGenerableEntity #5, #4, INIT_X_FIREBALLS_V, #60 , #0, #sys_ai_behaviour_straight_forward

    DefineGenerableEntity #30, #4, INIT_X_FIREBALLS_V, #120 , #0, #sys_ai_behaviour_straight_forward
    DefineGenerableEntity #1, #4, INIT_X_FIREBALLS_V, #70 , #0, #sys_ai_behaviour_straight_forward

    DefineGenerableEntity #30, #3, INIT_X_FIREBALLS_H, #70 , #0, #sys_ai_behaviour_straight_forward

    DefineGenerableEntity #10, #4, INIT_X_FIREBALLS_V, #70 , #0, #sys_ai_behaviour_straight_forward

    DefineGenerableEntity #20, #3, INIT_X_FIREBALLS_H, #110 , #0, #sys_ai_behaviour_straight_forward
    
    DefineGenerableEntity #10, #4, INIT_X_FIREBALLS_V, #20 , #0, #sys_ai_behaviour_straight_forward
    DefineGenerableEntity #1, #4, INIT_X_FIREBALLS_V, #70 , #0, #sys_ai_behaviour_straight_forward

    DefineGenerableEntity #10, #11, INIT_X_EXTRA_LIFE, #150 , #0, #sys_ai_behaviour_straight_forward_difficult

    ; Small break
    DefineGenerableEntity #150, #4, INIT_X_FIREBALLS_V, #120 , #0, #sys_ai_behaviour_straight_forward
    DefineGenerableEntity #1, #4, INIT_X_FIREBALLS_V, #70 , #0, #sys_ai_behaviour_straight_forward

    DefineGenerableEntity #20, #2, INIT_X_BIG_FIREBALL, #40 , #0, #sys_ai_behaviour_up_down

    DefineGenerableEntity #5, #3, INIT_X_FIREBALLS_H, #110 , #0, #sys_ai_behaviour_straight_forward
    DefineGenerableEntity #10, #4, INIT_X_FIREBALLS_V, #70 , #0, #sys_ai_behaviour_straight_forward

    ;; Small break
    DefineGenerableEntity #100, #4, INIT_X_FIREBALLS_V, #120 , #0, #sys_ai_behaviour_straight_forward
    DefineGenerableEntity #1, #4, INIT_X_FIREBALLS_V, #40 , #0, #sys_ai_behaviour_straight_forward


    DefineGenerableEntity #40, #12, INIT_X_FIREBALLS_H_LARGER, #70 , #0, #sys_ai_behaviour_straight_forward
    DefineGenerableEntity #1, #12, INIT_X_FIREBALLS_H_LARGER, #120 , #0, #sys_ai_behaviour_straight_forward

    DefineGenerableEntity #40, #12, INIT_X_FIREBALLS_H_LARGER, #90 , #0, #sys_ai_behaviour_up_down
    DefineGenerableEntity #1, #12, INIT_X_FIREBALLS_H_LARGER, #140 , #0, #sys_ai_behaviour_up_down

    .db #0

;; Final level
generable_combination_5::

    ;; [ (Counter, generable_entity, x, y, ai_counter, ai_func ), (...), ... ,(0) ]
    ;; Cross up-down bigs with small runs
    DefineGenerableEntity #20,                  #2, INIT_X_BIG_FIREBALL,    #(SCREEN_BOTTOM_BOUND-SCREEN_TOP_BOUND-SPRT_BIG_FIREBALL_H)/2+SCREEN_TOP_BOUND ,                #0, #sys_ai_behaviour_down_up
    DefineGenerableEntity #SPRT_BIG_FIREBALL_W, #2, INIT_X_BIG_FIREBALL,    #(SCREEN_BOTTOM_BOUND-SCREEN_TOP_BOUND-SPRT_BIG_FIREBALL_H)/2+SCREEN_TOP_BOUND ,                #0, #sys_ai_behaviour_up_down
    DefineGenerableEntity #5  ,                 #0 ,INIT_X_SMALL_FIREBALL,  #SCREEN_BOTTOM_BOUND-SPRT_SMALL_FIREBALL_H-13,                                                  #0, #sys_ai_behaviour_stop_and_run
    DefineGenerableEntity #5  ,                 #0 ,INIT_X_SMALL_FIREBALL,  #SCREEN_TOP_BOUND+13,                                                                           #0, #sys_ai_behaviour_stop_and_run

    ;; Forward horit with up-down balls
    DefineGenerableEntity #100, #0, INIT_X_SMALL_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_SMALL_FIREBALL_H-55 ,                                          #0, #sys_ai_behaviour_up_down
    DefineGenerableEntity #20, #0, INIT_X_SMALL_FIREBALL,  #SCREEN_BOTTOM_BOUND-SPRT_SMALL_FIREBALL_H-65 ,                                          #0, #sys_ai_behaviour_down_up

    DefineGenerableEntity #1  , #12, INIT_X_FIREBALLS_H_LARGER  , #SCREEN_TOP_BOUND+10,                                    #0, #sys_ai_behaviour_straight_forward
    DefineGenerableEntity #1   , #12, INIT_X_FIREBALLS_H_LARGER , #SCREEN_BOTTOM_BOUND-SPRT_FIREBALLS_H_LARGER_0_H-10,     #0, #sys_ai_behaviour_straight_forward

    DefineGenerableEntity #30, #0, INIT_X_SMALL_FIREBALL,  #SCREEN_BOTTOM_BOUND-SPRT_SMALL_FIREBALL_H-65 ,                                          #0, #sys_ai_behaviour_follow_player

    ;; Small follow
    DefineGenerableEntity #50, #0, INIT_X_SMALL_FIREBALL,  #SCREEN_TOP_BOUND ,                                          #0, #sys_ai_behaviour_follow_player
    DefineGenerableEntity #1,  #0, INIT_X_SMALL_FIREBALL,  #SCREEN_BOTTOM_BOUND-SPRT_SMALL_FIREBALL_H ,                 #0, #sys_ai_behaviour_follow_player

    ;; Big runs
    DefineGenerableEntity #50, #2, INIT_X_BIG_FIREBALL, #(SCREEN_BOTTOM_BOUND-SCREEN_TOP_BOUND)/2+SCREEN_TOP_BOUND, #0, #sys_ai_behaviour_stop_and_run
    DefineGenerableEntity #1, #2, INIT_X_BIG_FIREBALL, #SCREEN_TOP_BOUND+20,                                        #0, #sys_ai_behaviour_stop_and_run
    DefineGenerableEntity #1, #2, INIT_X_BIG_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_BIG_FIREBALL_H,                    #0, #sys_ai_behaviour_stop_and_run
   
    ;; Meds rain
    DefineGenerableEntity #100  , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_MEDIUM_FIREBALL_H-40,  #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #20   , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_MEDIUM_FIREBALL_H,     #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #10   , #11, INIT_X_EXTRA_LIFE     , #SCREEN_TOP_BOUND+20 ,                           #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #20   , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_TOP_BOUND,                               #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #20   , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_MEDIUM_FIREBALL_H-80,  #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #20   , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_MEDIUM_FIREBALL_H-20,  #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #20   , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_TOP_BOUND+10,                            #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #20   , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_TOP_BOUND+50,                            #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #20   , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_MEDIUM_FIREBALL_H-40,  #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #20   , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_TOP_BOUND+30,                            #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #20   , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_MEDIUM_FIREBALL_H-50,  #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #10   , #11, INIT_X_EXTRA_LIFE     , #SCREEN_TOP_BOUND+70 ,                           #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #20   , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_TOP_BOUND+10,                            #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #20   , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_MEDIUM_FIREBALL_H-40,  #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #20   , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_TOP_BOUND+30,                            #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #20   , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_MEDIUM_FIREBALL_H-20,  #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #20   , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_TOP_BOUND+20,                            #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #20   , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_MEDIUM_FIREBALL_H-60,  #0, #sys_ai_behaviour_straight_forward_difficult
    
    ;; Bigs rain
    DefineGenerableEntity #100  , #2 , INIT_X_BIG_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_BIG_FIREBALL_H-40,      #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #20   , #2 , INIT_X_BIG_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_BIG_FIREBALL_H,         #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #20   , #2 , INIT_X_BIG_FIREBALL, #SCREEN_TOP_BOUND,                                #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #20   , #2 , INIT_X_BIG_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_BIG_FIREBALL_H-80,      #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #20   , #2 , INIT_X_BIG_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_BIG_FIREBALL_H-20,      #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #20   , #2 , INIT_X_BIG_FIREBALL, #SCREEN_TOP_BOUND+10,                             #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #20   , #2 , INIT_X_BIG_FIREBALL, #SCREEN_TOP_BOUND+50,                             #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #20   , #2 , INIT_X_BIG_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_BIG_FIREBALL_H-40,      #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #20   , #2 , INIT_X_BIG_FIREBALL, #SCREEN_TOP_BOUND+30,                             #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #20   , #2 , INIT_X_BIG_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_BIG_FIREBALL_H-50,      #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #20   , #2 , INIT_X_BIG_FIREBALL, #SCREEN_TOP_BOUND+10,                             #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #20   , #2 , INIT_X_BIG_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_BIG_FIREBALL_H-40,      #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #20   , #2 , INIT_X_BIG_FIREBALL, #SCREEN_TOP_BOUND+30,                             #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #20   , #2 , INIT_X_BIG_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_BIG_FIREBALL_H-20,      #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #20   , #2 , INIT_X_BIG_FIREBALL, #SCREEN_TOP_BOUND+20,                             #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #20   , #2 , INIT_X_BIG_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_BIG_FIREBALL_H-60,      #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #5    , #11, INIT_X_EXTRA_LIFE  , #SCREEN_BOTTOM_BOUND-SPRT_BIG_FIREBALL_H-80,      #0, #sys_ai_behaviour_straight_forward_difficult
    
    ;; YOU WONT ESCAPE!
    DefineGenerableEntity #50  , #13, #9, #100, #MESSAGE_TIME, #sys_ai_behaviour_kill_alarm_message

    ;; Meds fast rain
    DefineGenerableEntity #100  , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_MEDIUM_FIREBALL_H-40,  #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #10   , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_MEDIUM_FIREBALL_H,     #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #10   , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_TOP_BOUND,                               #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #10   , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_MEDIUM_FIREBALL_H-80,  #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #10   , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_MEDIUM_FIREBALL_H-20,  #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #10   , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_TOP_BOUND+10,                            #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #10   , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_TOP_BOUND+50,                            #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #10   , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_MEDIUM_FIREBALL_H-40,  #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #10   , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_TOP_BOUND+30,                            #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #10   , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_MEDIUM_FIREBALL_H-50,  #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #10   , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_TOP_BOUND+10,                            #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #10   , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_MEDIUM_FIREBALL_H-40,  #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #10   , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_TOP_BOUND+30,                            #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #10   , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_MEDIUM_FIREBALL_H-20,  #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #10   , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_TOP_BOUND+20,                            #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #10   , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_MEDIUM_FIREBALL_H-60,  #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #10   , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_TOP_BOUND+20,                            #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #10   , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_MEDIUM_FIREBALL_H-20,  #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #10   , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_TOP_BOUND+30,                            #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #10   , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_MEDIUM_FIREBALL_H-40,  #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #10   , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_TOP_BOUND+10,                            #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #10   , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_MEDIUM_FIREBALL_H-50,  #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #10   , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_TOP_BOUND+30,                            #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #10   , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_MEDIUM_FIREBALL_H-40,  #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #10   , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_TOP_BOUND+50,                            #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #10   , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_TOP_BOUND+10,                            #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #10   , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_MEDIUM_FIREBALL_H-20,  #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #10   , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_MEDIUM_FIREBALL_H-80,  #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #10   , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_TOP_BOUND,                               #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #10   , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_MEDIUM_FIREBALL_H,     #0, #sys_ai_behaviour_straight_forward_difficult
    DefineGenerableEntity #10   , #1 , INIT_X_MEDIUM_FIREBALL, #SCREEN_BOTTOM_BOUND-SPRT_MEDIUM_FIREBALL_H-40,  #0, #sys_ai_behaviour_straight_forward_difficult
    
    ;; NOOO!!
    DefineGenerableEntity #20  , #14, #29, #100, #MESSAGE_TIME, #sys_ai_behaviour_kill_alarm_message

    .db 0

;======================================================================================================
;  Starts a new generation combination
;
;  Inputs:
;      -
;  Outputs:
;      - 
;
sys_generator_start_new_combination::
    
    
    ld ix, (current_level)

    ;; Obtengo la dirección de la combinación
    ld  l,  (ix)
    ld  h,  1(ix)

    ;; Guardo el combination al combination_generating
    ld (combination_generating), hl

    ;; Guardo el counter del generator
    ld a, (hl)
    ld (generator_counter), a

    ;; Pongo a 1 que hay un combination en proceso
    ld a, #1
    ld (theres_combination_in_process), a

    ret

;======================================================================================================
;  Generates a the following element in the combination in process
;
;  Inputs:
;      -
;  Outputs:
;      -
;
generate_next_element_in_combination:
    call man_entity_create  ;; DE -> entity generado
    push de     ;; Guardo el entity generado

    ld ix, (combination_generating)
    
    ;; Obtengo indice del tmpl a generar
    ld a, gen_comb_entIdx(ix)
    add a, a

    ;; Obtengo puntero a la lista de tmpl
    ld hl, #generable_entities
    ld b, #0
    ld c, a
    add hl, bc

    ;; Obtengo el tmpl
    ld c, (hl)
    inc hl
    ld b, (hl)

    push bc
    pop hl

    ;; DE has the position of the entity created
    ;; HL has te tmpl to create
    ld      bc, #sizeof_e       ; SIZE
    call cpct_memcpy_asm

    ld ix, (combination_generating) ;; Obtengo generating para transferir los datos al entity creado
    pop iy          ;; Recupero entity creado para editarlo

    ;; Actualizo entity creado
    ld a, gen_comb_entX(ix)     ;; Gets X
    ld e_x(iy), a               ;; Guardo X
    
    ld a, gen_comb_entY(ix)     ;; Gets Y
    ld e_y(iy), a               ;; Guardo Y

    ld a, gen_comb_aiCounter(ix)        ;; Gets ai counter
    ld e_behaviour_count(iy),   a       ;; Guardo ai function

    ld c, gen_comb_aiFunc(ix)           ;; Gets ai function
    ld e_behaviour_func(iy),    c       ;; Guardo ai function

    ld a, gen_comb_aiFunc+1(ix)         ;; Gets ai function
    ld e_behaviour_func+1(iy),  a       ;; Guardo ai function


    ;; Comparo las dos partes del puntero a la función para ver si hay AI
    or a, c

    ;; Si no la hay, salto
    jr Z, no_ai_function

        ;; Si hay una función, inserto el componente ai
        ld a, e_cmps(iy)
        or #e_cmp_ai
        ld e_cmps(iy), a

    no_ai_function:

    ; ld a, (enemy_speed)
    ; ld e_vx(iy), a  ;; Cargo VX

    ret





;======================================================================================================
;  Initializes generator values
;
;  Inputs:
;      -
;  Outputs:
;      -
;
sys_generator_init::

    ld hl, #generator_counter
    ld (hl), #0

    ld hl, #theres_combination_in_process
    ld (hl), #0

    ld hl, #0
    ld (combination_generating), hl

    ret


;; IX: Combination to generate
sys_generator_update::


    ;; Compruebo si hay una combinación generandose en proceso
    ld a, (theres_combination_in_process)
    or a
    ret Z   ;; If there's no combination to generate, ret


    ;; Decremento el counter
    ld hl, #generator_counter
    dec (hl)

    ;; Si el counter no es 0, no genero nada (fin)
    ret NZ
    
    ;; Si el counter es 0, compruebo el free space
    call man_entity_freeSpace
    or a
    jr NZ, generate_new_element_generator_generate
        ;; Si no queda espacio, incremento counter y salgo
        ld hl, #generator_counter
        inc (hl)
        ret

    generate_new_element_generator_generate:
    
    ;; Si el counter es 0, genero
    call generate_next_element_in_combination
    
    ;; Obtengo el siguiente generate
    ld hl, (combination_generating)
    ld bc, #SINGLE_GENERATE_SIZE
    add hl, bc
    ld (combination_generating), hl
    
    ;; Obtengo el counter
    ld a, (hl)

    ;; Guardo el counter
    ld (generator_counter), a

    ;; Comprueba que no sea 0 el counter
    or a
    ret NZ

    ;; Si es 0, fin de combinación (pongo is_combination_in_process a 0) 
    ld a, #0
    ld (theres_combination_in_process), a

    ;; Será el propio levels quien vea que ya no hay nada generandose para pasar al siguiente nivel

    ret