; MIT License

; Copyright (C) 2022 Z80Enjoyers / Luis Jesús Marhuenda Tendero (@ljmt1) / Alberto Vegara Belmonte (@avb52-ua) / Samuel Benjamín Oliva Bulpitt (@samuel0122)

; Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal 
; in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
; copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

; The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS 
; FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
; WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

;;======================================================================================================
.module Input_System
;;======================================================================================================
.area _CODE
;;======================================================================================================
.include "cpctelera.h.s"
.include "cpct_functions.inc"
.include "man/entity.h.s"
.include "input.h.s"
;;======================================================================================================

;;======================================================================================================
;;  Private Values
;;
    PLAYER_SPEED_X = 1
    PLAYER_SPEED_Y = 3

;; Keybinds
key_actions:
    .dw Key_O,          move_left       ; DW because Key_O, are symbols of 4 bytes
    .dw Key_P,          move_right      ; DW because Key_P, are symbols of 4 bytes
    .dw Key_A,          drop            ; DW because Key_Q, are symbols of 4 bytes
    .dw Key_Q,          jump            ; DW because Key_W, are symbols of 4 bytes
    .dw Joy0_Left,      move_left       ; DW because Key_O, are symbols of 4 bytes
    .dw Joy0_Right,     move_right      ; DW because Key_P, are symbols of 4 bytes
    .dw Joy0_Up,        jump            ; DW because Key_Q, are symbols of 4 bytes
    .dw Joy0_Down,      drop            ; DW because Key_W, are symbols of 4 bytes
    .dw 0                               ; Exit value for the loop

move_left:
    ld    e_vx(ix), #-PLAYER_SPEED_X
    ret
move_right:
    ld    e_vx(ix), #PLAYER_SPEED_X
    ret
jump:
    ld    e_vy(ix), #-PLAYER_SPEED_Y-1
    ret
drop:
    ld    e_vy(ix), #PLAYER_SPEED_Y-1
    ret

;;======================================================================================================
;;  Check the keyboard state, and update physics
;;  based on it, i.e. move the entity
;;
;;  Inputs:
;;      - IX: Pointer to the entity
;;
sys_input_check_keyboard_and_update_player::
    ;; Reset velocity of the player
    ld      e_vx(ix), #0
    ld      e_vy(ix), #0

    ;; Check keyboard
    call    cpct_scanKeyboard_f_asm ;; No input

    ;; Key-Action Check-Call loop
    ld      iy, #key_actions-4 ;; Start before the first key in order to make te increment at the begining of the loop
    
    loop_keys:
        ld  bc, #4  ;; / IY += 4 (Next Key)
        add iy, bc  ;; \

        ld  l, 0(iy)    ;; / HL = Next key
        ld  h, 1(iy)    ;; \

        ;; Check if key==null TODO: make macro for this check
        ld  a, l                ;; / A = H | L
        or  h                   ;; \
        jr  z, end_loop_keys    ;; If A=0, Key=null, ret

        ;; Check if key is pressed
        call cpct_isKeyPressed_asm  ;; Input-> HL(key)
        jr  z, loop_keys

        ;; Key is pressed, execute action
        ld      hl, #loop_keys  ;; / Return to the begin of the loop when return from the action function pointer
        push    hl              ;; \

        ld      l, 2(iy)    ;; / HL = action function pointer
        ld      h, 3(iy)    ;; \
        jp      (hl)        ;; Jump to actino function
        ;; jr loop_keys, implicit on function ret
    
    end_loop_keys:

    ret


;;======================================================================================================
;;  INPUT UPDATE
;; 
;;  Outputs:
;;
sys_input_update::
    ; TODO: IMPLEMENT "man_entity_get_from_idx"
    ; ;; Get Entity 0 (Player) in IX
    ; xor     a
    ; call man_entity_get_from_idx

    ld    hl, #sys_input_check_keyboard_and_update_player
    ld    b, #e_cmp_input
    call man_entity_forall_matching

    ret


sys_input_player_go_down_and_right::
    ld    e_vx(ix), #PLAYER_SPEED_X
    ld    e_vy(ix), #PLAYER_SPEED_Y-1
    ret