; MIT License

; Copyright (C) 2022 Z80Enjoyers / Luis Jesús Marhuenda Tendero (@ljmt1) / Alberto Vegara Belmonte (@avb52-ua) / Samuel Benjamín Oliva Bulpitt (@samuel0122)

; Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal 
; in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
; copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

; The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS 
; FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
; WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

;;======================================================================================================
.module Physics_System
;;======================================================================================================
.area _CODE
;;======================================================================================================
.include "physics.h.s"
.include "cpctelera.h.s"
.include "cpct_functions.inc"
.include "man/entity.h.s"
.include "man/macros.h.s"
.include "assets/sprites.inc"
;;======================================================================================================
;======================================================================================================
;;  PHYSICS UPDATE
;; 
;;  Outputs:
;;      - HL: POINTER TO THE SYS_PHYSICS_UPDATE_ONE_ENTITY
;;
sys_physics_update::

    ld    hl, #sys_physics_update_one_entity
    ld    b, #e_cmp_physics
    call  man_entity_forall_matching

    ret

;======================================================================================================
;;  Update physics of the player
;;
;;  Inputs:
;;      - IX: Pointer to the entity to be updated
;;
sys_physics_update_player:
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;;; HORIZONTAL MOVES
    ;; Check if we are on the left bound
    ;; La primera comprobación es para ver en qué posición está. Si está en el borde, compruebo velocidad

    CheckLeftBound ix, is_not_at_the_left_bound
        ;; Esta segunda comprobación es para la velocidad. Si estoy en el borde y "quiero salirme", pongo la velocidad a 0
        CheckVxNegative ix, end_check_HoritzontalMoves
            ld      e_vx(ix),   #0
            ld      e_x(ix),    #0
            jr      end_check_HoritzontalMoves
    is_not_at_the_left_bound:

    ;; Check if we are on the right bound
    ;; La primera comprobación es para ver en qué posición está. Si está en el borde, compruebo velocidad
    CheckRightBound ix, end_check_HoritzontalMoves
        CheckVxPositive ix, end_check_HoritzontalMoves
    
        ;; Esta segunda comprobación es para la velocidad. Si estoy en el borde y "quiero salirme", pongo la velocidad a 0
            ld      e_vx(ix),   #0
            ld      e_x(ix),    #SCREEN_WITDH - SPRT_PLAYER_W
    end_check_HoritzontalMoves:

    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;;; VERTICAL MOVES
    ;; Check if we are going above of the top bound
    ;; Apply basic gravity
    inc     e_vy(ix)

    CheckTopBound ix, is_not_at_the_top_bound
        CheckVyNegative ix, end_check_VerticalMoves
            ld      e_vy(ix),   #0
            ld      e_y(ix),    #SCREEN_TOP_BOUND
            jr      end_check_VerticalMoves
    is_not_at_the_top_bound:

    ;; Check if we are at bottom bound
    CheckBottomBound ix, end_check_VerticalMoves
        CheckVyPositive ix, end_check_VerticalMoves
            ld      e_vy(ix),   #0
            ld      e_y(ix),    #SCREEN_BOTTOM_BOUND - SPRT_PLAYER_H
    
    end_check_VerticalMoves:
    
    ret

;======================================================================================================
;  Update physics of a given entity
;
;  Inputs:
;      - IX: Pointer to the entity to be updated
;
sys_physics_update_one_entity::

    ;; Checks if it's a player
    ld a, e_type(ix)
    and #e_type_player

    jr z, not_player
        ;; If it's player, update player
        call sys_physics_update_player
    not_player:

    ;; Updates position
    ld      a, e_x(ix)  ;; /
    add     e_vx(ix)    ;;| e->x += vx
    ld      e_x(ix), a  ;; \
    
    ld      a, e_y(ix)  ;; /
    add     e_vy(ix)    ;;| e->y += vy
    ld      e_y(ix), a  ;; \


    ;; Checks if X is negative (it's gone over the left edge)
    ld a, e_x(ix)
    and #0x80

    ret Z   ;; If it hasn't, leave it
    
    ;; If it has, set it to be destroyed (or draw it partially)
    call man_entity_set4destroy
    
    ret




;======================================================================================================
;;  Updates player physics for the win animation
;;
sys_physics_kill_player_if_right_4animation::

    ;; Checks player X position is on right bound
    ld ix, #entities
    ld a, e_x(ix)

    cp #SCREEN_WITDH-SPRT_PLAYER_W

    ;; If not in right bound, ignore
    ret NZ

    ;; If it's in right bound, kill
    call man_entity_set4destroy
    ret